@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # BioSTEAM: The Biorefinery Simulation and Techno-Economic Analysis Modules
-# Copyright (C) 2020-2021, Yoel Cortes-Pena <yoelcortes@gmail.com>
+# Copyright (C) 2020-2023, Yoel Cortes-Pena <yoelcortes@gmail.com>
 # 
 # This module is under the UIUC open-source license. See 
 # github.com/BioSTEAMDevelopmentGroup/biosteam/blob/master/LICENSE.txt
@@ -22,11 +22,21 @@ def test_reaction():
     assert not reaction.stoichiometry.any()
     
     # Test math cycles, making sure they balance out
-    reaction = tmo.Reaction('2H2O -> 2H2 + O2', reactant='H2O',
-                            correct_atomic_balance=True, X=0.5)
+    single_phase_reaction = reaction = tmo.Reaction(
+        '2H2O -> 2H2 + O2', reactant='H2O',
+        correct_atomic_balance=True, X=0.5
+    )
     same_reaction = reaction.copy()
     reaction += same_reaction
     reaction -= same_reaction
+    reaction += 0 # Working with 0 allows for pythonic sum method
+    reaction -= 0
+    reaction += None # While this might not make much pythonic sense, it was voted by biosteam users for convinience
+    reaction -= None
+    reaction = reaction + 0 
+    reaction = reaction - 0
+    reaction = reaction + None 
+    reaction = reaction - None
     assert_allclose(reaction.X, same_reaction.X)
     reaction *= 2.
     reaction /= 2.
@@ -37,8 +47,10 @@ def test_reaction():
     assert_allclose(negative_reaction.X, -1.)
     
     # Test errors with incompatible phases
-    reaction = tmo.Reaction('H2O,l -> H2,g + O2,g', reactant='H2O',
-                            correct_atomic_balance=True, X=0.7)
+    multi_phase_reaction = reaction = tmo.Reaction(
+        'H2O,l -> H2,g + O2,g', reactant='H2O',
+        correct_atomic_balance=True, X=0.7
+    )
     stream = tmo.MultiStream(None, l=[('H2O', 10)], phases='lL')
     with pytest.raises(ValueError): reaction(stream)
     
@@ -48,6 +60,43 @@ def test_reaction():
                              phases='gl')
     with pytest.raises(tmo.exceptions.UndefinedChemical): reaction(stream)
     
+    # Test errors with chemical groups
+    tmo.settings.chemicals.define_group('CriticalGases', ['H2', 'O2'])
+    with pytest.raises(ValueError): 
+        reaction = tmo.Reaction('H2O -> CriticalGases', reactant='H2O',
+                                correct_atomic_balance=True, X=0.7)
+        
+    with pytest.raises(tmo.exceptions.UndefinedChemical): 
+        reaction = tmo.Reaction('H2O -> UnknownChemical', reactant='H2O',
+                                correct_atomic_balance=True, X=0.7)
+        
+    # Test special methods
+    
+    # Test product_yield with and without multiple phases
+    assert_allclose(single_phase_reaction.product_yield('O2'), 0.25)
+    assert_allclose(multi_phase_reaction.product_yield('O2'), 0.35)
+    assert_allclose(single_phase_reaction.product_yield('H2'), 0.5)
+    assert_allclose(multi_phase_reaction.product_yield('H2'), 0.7)
+    assert_allclose(single_phase_reaction.product_yield('O2', basis='wt'), 0.44405082796381734)
+    assert_allclose(multi_phase_reaction.product_yield('O2', basis='wt'), 0.6216711591493442)
+    
+    # Test product_yield setter with and without multiple phases
+    single_phase_reaction.product_yield('O2', product_yield=0.1)
+    multi_phase_reaction.product_yield('O2', product_yield=0.1)
+    assert_allclose(single_phase_reaction.product_yield('O2'), 0.1)
+    assert_allclose(multi_phase_reaction.product_yield('O2'), 0.1)
+    
+    single_phase_reaction.product_yield('H2', product_yield=0.1)
+    multi_phase_reaction.product_yield('H2', product_yield=0.1)
+    assert_allclose(single_phase_reaction.product_yield('H2'), 0.1)
+    assert_allclose(multi_phase_reaction.product_yield('H2'), 0.1)
+    
+    single_phase_reaction.product_yield('O2', basis='wt', product_yield=0.1)
+    multi_phase_reaction.product_yield('O2', basis='wt', product_yield=0.1)
+    assert_allclose(single_phase_reaction.product_yield('O2', basis='wt'), 0.1)
+    assert_allclose(multi_phase_reaction.product_yield('O2', basis='wt'), 0.1)
+    
+    
 def test_reaction_enthalpy_balance():
     # Combustion; ensure heat of gas phase reaction without sensible heats is 
     # the lower heating value
