@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # BioSTEAM: The Biorefinery Simulation and Techno-Economic Analysis Modules
-# Copyright (C) 2020-2021, Yoel Cortes-Pena <yoelcortes@gmail.com>
+# Copyright (C) 2020-2023, Yoel Cortes-Pena <yoelcortes@gmail.com>
 # 
 # This module is under the UIUC open-source license. See 
 # github.com/BioSTEAMDevelopmentGroup/biosteam/blob/master/LICENSE.txt
@@ -11,6 +11,21 @@ import pytest
 import thermosteam as tmo
 from numpy.testing import assert_allclose
 
+def test_registration_bypass():
+    tmo.settings.set_thermo(['Water'], cache=True)
+    s = tmo.Stream('.s1', Water=1)
+    assert s.ID == 's1'
+    assert s not in s.registry
+    s = tmo.Stream('s.1', Water=1)
+    assert s.ID == 's.1'
+    assert s not in s.registry
+
+def test_registration_alias():
+    tmo.settings.set_thermo(['Water'], cache=True)
+    s = tmo.Stream('s1', Water=1)
+    s.register_alias('stream_1')
+    assert s.registry.stream_1 is s.registry.s1 is s
+
 def test_vlle():
     tmo.settings.set_thermo(['Water', 'Ethanol', 'Octane'], cache=True)
     s = tmo.Stream(None, Water=1, Ethanol=1, Octane=2, vlle=True, T=350)
@@ -36,6 +51,10 @@ def stream_methods():
     s2 = tmo.Stream(None, Water=100, Ethanol=100, phase='g')
     s3 = tmo.Stream.sum([s1, s2], vle=True)
     assert_allclose([s3.T, s3.vapor_fraction], [356.42348688162764, 0.9159452891657182])
+    s1.P = 101325 * 5
+    s2.P = 101325 * 2
+    s3.mix_from([s1, s2], vle=True)
+    assert s3.P == min([s1.P, s2.P])
 
 def test_critical():
     tmo.settings.set_thermo(['CO2', 'O2', 'CH4'], cache=True)
@@ -263,6 +282,44 @@ def test_stream_property_cache():
     values = [getprop(i, T, True) for i, T in zip(IDs, Ts)]
     values_cache = [getprop(i, T, False) for i, T in zip(IDs, Ts)]
     assert_allclose(values, values_cache)
+    
+    ### Same test but for multistreams
+    s = tmo.Stream(None, Water=2, T=299.15)
+    s.phases = ('l', 'g')
+    
+    # Cache is being used to retrieve flow properties; make sure results are scaled properly
+    prop_IDs = ['h', 'Cn']
+    flow_IDs = ['H', 'C']
+    prop_values = [getattr(s, i) for i in prop_IDs]
+    flow_values = [getattr(s, i) for i in flow_IDs]
+    assert_allclose(flow_values, [i*s.F_mol for i in prop_values])
+    
+    # Make sure cache is being cleared when conditions are changed between evaluations.
+    # This is done by computing a property at one temperature, another property at new temperature,
+    # then the first property at the new temperature.
+    def getprop(name, T, clear_cache):
+        s.T = T
+        if clear_cache: s._property_cache.clear()
+        return getattr(s, name)
+    
+    IDs = ['H', 'C', 'H']
+    Ts = [298.15, 310, 310]
+    values = [getprop(i, T, True) for i, T in zip(IDs, Ts)]
+    values_cache = [getprop(i, T, False) for i, T in zip(IDs, Ts)]
+    assert_allclose(values, values_cache)
+
+def test_mixing_balance():
+    tmo.settings.set_thermo(['Water'], cache=True)
+    ms = tmo.MultiStream(None, l=[('Water', 1)], g=[('Water', 2)])
+    streams = [ms['g'], ms['l'], ms['l']]
+    total_flow = sum([i.imol['Water'] for i in streams])
+    ms.mix_from(streams)
+    assert ms.imol['Water'] == total_flow
+    ms.imol['g', 'Water'] = 0.
+    streams = [ms, ms['l'], ms['l']]
+    total_flow = sum([i.imol['Water'] for i in streams])
+    ms['l'].mix_from(streams)
+    assert ms['l'].imol['Water'] == total_flow
 
 def test_mixture():
     tmo.settings.set_thermo(['Water'], cache=True)
@@ -275,12 +332,36 @@ def test_mixture():
     Ts = s5.mixture.solve_T_at_SP(phase=s5.phase, mol=s5.mol, S=s5.S, T_guess=s5.T, P=s5.P)
     assert_allclose(T_expected, Ts, rtol=1e-3)
     pass
+
+def test_vle_critical_pure_component():
+    N2 = tmo.Chemical('N2', cache=True)
+    tmo.settings.set_thermo([N2])
+    s = tmo.Stream(None, N2=1)
+    
+    # Under critical point, liquid is fine
+    s.vle(T=N2.Tc - 1e-6, P=2 * N2.Pc)
+    assert s.phase == 'l'
+    
+    # Beyond critical point, always model as a gas
+    s.vle(T=N2.Tc, P=0.5 * N2.Pc)
+    assert s.phase == 'g'
+    s.vle(T=N2.Tc, P=2 * N2.Pc)
+    assert s.phase == 'g'
+    s.vle(P=N2.Pc, H=s.H + 10)
+    assert s.phase == 'g'
+    s.vle(P=N2.Pc, S=s.S + 10)
+    assert s.phase == 'g'
+    
     
 if __name__ == '__main__':
+    test_registration_bypass()
+    test_registration_alias()
     test_stream()
     test_multistream()
+    test_mixing_balance()
     test_vlle()
     stream_methods()
     test_stream_property_cache()
+    test_vle_critical_pure_component()
     test_critical()
     test_mixture()
\ No newline at end of file
