@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # BioSTEAM: The Biorefinery Simulation and Techno-Economic Analysis Modules
-# Copyright (C) 2020-2021, Yoel Cortes-Pena <yoelcortes@gmail.com>
+# Copyright (C) 2020-2023, Yoel Cortes-Pena <yoelcortes@gmail.com>
 # 
 # This module is under the UIUC open-source license. See 
 # github.com/BioSTEAMDevelopmentGroup/biosteam/blob/master/LICENSE.txt
@@ -42,7 +42,13 @@ from .free_energy import (
     EnthalpyRefSolid, EnthalpyRefLiquid, EnthalpyRefGas,
     EntropyRefSolid, EntropyRefLiquid, EntropyRefGas,
     ExcessEnthalpyRefSolid, ExcessEnthalpyRefLiquid, ExcessEnthalpyRefGas,
-    ExcessEntropyRefSolid, ExcessEntropyRefLiquid, ExcessEntropyRefGas
+    ExcessEntropyRefSolid, ExcessEntropyRefLiquid, ExcessEntropyRefGas,
+    Excess_Solid_Enthalpy_Ref_Solid,
+    Excess_Solid_Entropy_Ref_Solid,
+    Excess_Liquid_Enthalpy_Ref_Liquid,
+    Excess_Liquid_Entropy_Ref_Liquid,
+    Excess_Gas_Enthalpy_Ref_Gas,
+    Excess_Gas_Entropy_Ref_Gas
 )
 from .equilibrium.unifac import (
     DDBST_UNIFAC_assignments, 
@@ -83,7 +89,7 @@ from thermo import (
 
 __all__ = ('Chemical',)
 
-# %% Temporarilly here
+# %% Temporarily here
 
 # TODO: Make a data table for new additions (not just for common sugars).
 # Source: PubChem
@@ -107,6 +113,25 @@ class CompressibilityFactor:
     def __repr__(self):
         return f"{type(self).__name__}(self.V)"
 
+try:
+    from CoolProp.CoolProp import PropsSI
+except:
+    pass
+else:
+    class CoolPropFreeEnergy:
+        __slots__ = ('_ref', 'cache', 'name', 'CAS', 'MW')
+        
+        def __init__(self, CAS, name, MW):
+            self.CAS = CAS
+            self.name = name
+            self.cache = (None, None)
+            self._ref = PropsSI(self.name, 'T', 298.15, 'P', 101325., self.CAS) / 1000. * MW
+            self.MW = MW
+        
+        def __call__(self, T, P):
+            return PropsSI(self.name, 'T', T, 'P', P, self.CAS) * self.MW / 1000. - self._ref
+
+
 # %% Filling missing properties
 
 def get_chemical_data(chemical):
@@ -286,7 +311,7 @@ class Chemical:
     sigma : float or function(T), optional
         Surface tension model [N/m] as a function of temperature [K].
     epsilon : float or function(T), optional
-        Relative permitivity model [-] as a function of temperature [K].
+        Relative permittivity model [-] as a function of temperature [K].
     Psat : float or function(T), optional
         Vapor pressure model [N/m] as a function of temperature [K].
     Hvap : float or function(T), optional
@@ -400,7 +425,7 @@ class Chemical:
     Phase dependent properties have attributes with model handles for each phase:
 
     >>> Water.V
-    <PhaseTPHandle(phase, T, P) -> V [m^3/mol]>
+    <PhaseTPHandle(phase, T, P=None) -> V [m^3/mol]>
     >>> # Water.V.l
     >>> # VolumeLiquid(CASRN="7732-18-5", MW=18.01528, Tb=373.124, Tc=647.14, Pc=22048320.0, Vc=5.6-05, Zc=0.2294727, omega=0.344, dipole=1.85, Psat=VaporPressure(CASRN="7732-18-5", Tb=373.124, Tc=647.14, Pc=22048320.0, omega=0.344, extrapolation="AntoineAB|DIPPR101_ABC", method="WAGNER_MCGARRY"), eos=[PR(Tc=647.14, Pc=22048320.0, omega=0.344, T=298.15, P=101325.0)], extrapolation="constant", method="VDI_PPDS", method_P="COSTALD_COMPRESSED", tabular_extrapolation_permitted=True)
     
@@ -423,15 +448,8 @@ class Chemical:
 
     Choose what model to use through the `method` attribute:
     
-    >>> list(sorted(Water.Cn.l.all_methods))
-    ['CRCSTD',
-     'DADGOSTAR_SHAW',
-     'POLING_CONST',
-     'ROWLINSON_BONDI',
-     'ROWLINSON_POLING',
-     'USER_METHOD',
-     'WEBBOOK_SHOMATE',
-     'ZABRANSKY_SPLINE_C']
+    >>> list(sorted(Water.Cn.l.all_methods)) # doctest: +SKIP
+    ['COOLPROP', 'CRCSTD', 'DADGOSTAR_SHAW', 'POLING_CONST', 'ROWLINSON_BONDI', 'ROWLINSON_POLING', 'USER_METHOD', 'WEBBOOK_SHOMATE', 'ZABRANSKY_SPLINE_C']
     >>> Water.Cn.l.method = 'ZABRANSKY_SPLINE_C'
     
     .. note::
@@ -457,7 +475,7 @@ class Chemical:
     sigma(T) : 
         Surface tension [N/m].
     epsilon(T) : 
-        Relative permitivity [-]
+        Relative permittivity [-]
     S(phase, T, P) : 
         Entropy [J/mol].
     H(phase, T) : 
@@ -494,13 +512,15 @@ class Chemical:
                 default=False, phase=None, search_db=True, 
                 V=None, Cn=None, mu=None, Cp=None, rho=None,
                 sigma=None, kappa=None, epsilon=None, Psat=None,
-                Hvap=None, **data):
+                Hvap=None, method=None, **data):
         chemical_cache = cls.chemical_cache
-        if (cache or cls.cache) and ID in chemical_cache:
+        if cache is None: cache = cls.cache
+        if cache and ID in chemical_cache:
             if any([search_ID, eos, phase_ref, CAS, default, phase, 
                     V, Cn, mu, Cp, rho, sigma, kappa, epsilon, Psat, Hvap,
                     data]):
-                warn('cached chemical returned; additional parameters disregarded')
+                warn('cached chemical returned; additional parameters disregarded',
+                     RuntimeWarning)
             return chemical_cache[ID]
         ID = ID.strip()
         if not phase and len(ID) > 2 and ID[-2] == ',':
@@ -512,10 +532,9 @@ class Chemical:
         if search_db:
             metadata = pubchem_db.search(search_ID)
             data['metadata'] = metadata
-            self = cls.new(ID, metadata.CASs, eos, phase_ref, phase,
-                           **data)
+            self = cls.new(ID, metadata.CASs, eos, phase_ref, phase, free_energies=False, **data)
         else:
-            self = cls.blank(ID, CAS, phase_ref, phase=phase, **data)
+            self = cls.blank(ID, CAS, phase_ref, phase=phase, free_energies=False, **data)
         if phase:
             if mu: self._mu.add_method(mu)
             if Cn: self._Cn.add_method(Cn)
@@ -534,18 +553,61 @@ class Chemical:
         if epsilon: self._epsilon.add_method(epsilon)
         if Psat: self._Psat.add_method(Psat)
         if Hvap: self._Hvap.add_method(Hvap)
-        if default: self.default()
         if phase:
-            if Cp: self._Cn.add_method(Cp * self._MW)
-            if rho: self._V.add_method(fn.rho_to_V(rho, self._MW))
+            MW = self._MW
+            if Cp: self._Cn.add_method(Cp if MW is None else Cp * MW)
+            if rho: self._V.add_method(fn.rho_to_V(rho, 1.) if MW is None else fn.rho_to_V(rho, MW))
+        if default: self.default()
         if cache:
             chemical_cache[ID] = self
-            if len(chemical_cache) > 100:
+            if len(chemical_cache) > 1000:
                 for i in chemical_cache:
                     del chemical_cache[i]
                     break
+        if method: self.set_method(method)
+        self.reset_free_energies()
         return self
 
+    def set_method(self, method):
+        for name in _model_handles:
+            model_handle = getattr(self, name)
+            if method not in model_handle.all_methods: continue
+            model_handle.method = method
+        for name in _phase_handles:
+            obj = getattr(self, name)
+            if method in model_handle.all_methods: 
+                if isinstance(obj, PhaseHandle):
+                    for phase, model_handle in obj:
+                        model_handle.method = method
+                        
+                else:
+                    obj.method = method
+            if hasattr(model_handle, 'all_methods_P') and method in model_handle.all_methods_P: 
+                if isinstance(obj, PhaseHandle):
+                    for phase, model_handle in obj:
+                        model_handle.method_P = method
+                else:
+                    obj.method_P = method
+        
+    def use_coolprop_free_energies(self):
+        setattr = object.__setattr__
+        for name in ('_H', '_S'):
+            obj = getattr(self, name)
+            # Phase determined by coolprop
+            cpfe = CoolPropFreeEnergy(self.CAS, name.strip('_'), self.MW)
+            if isinstance(obj, PhaseHandle):
+                for phase, model_handle in obj: setattr(obj, phase, cpfe)
+            else:
+                setattr(obj, phase, cpfe)
+        # Excess energies are always included
+        f = lambda T, P: 0.
+        for name in ('_H_excess', '_S_excess'):
+            obj = getattr(self, name)
+            if isinstance(obj, PhaseHandle):
+                for phase, model_handle in obj: setattr(obj, phase, f)
+            else:
+                setattr(obj, phase, f)
+
     @classmethod
     def new(cls, ID, CAS, eos=None, phase_ref=None, phase=None, **data):
         """Create a new chemical from data without searching through
@@ -557,7 +619,7 @@ class Chemical:
 
     @classmethod
     def blank(cls, ID, CAS=None, phase_ref=None, phase=None,
-              formula=None, aliases=None, synonyms=None, **data):
+              formula=None, aliases=None, synonyms=None, free_energies=True, **data):
         """
         Return a new Chemical object without any thermodynamic models or data 
         (unless provided).
@@ -580,7 +642,7 @@ class Chemical:
         >>> from thermosteam import Chemical
         >>> Substance = Chemical.blank('Substance')
         >>> Substance.show()
-        Chemical: Substance (phase_ref=None)
+        Chemical: Substance (phase_ref='l')
         [Names]  CAS: Substance
                  InChI: None
                  InChI_key: None
@@ -645,19 +707,23 @@ class Chemical:
                            self._iscyclic_aliphatic, self._eos, self.has_hydroxyl,
                            self._Hfus)
         self._locked_state = None
-        if phase: self.at_state(phase)
         self._estimate_missing_properties(self.Psat)
-        self._init_energies(self._Cn, self._Hvap, self._Psat, self._Hfus, self._Sfus,
-                            self._Tm, self._Tb, self._eos, self._phase_ref)
+        if phase: 
+            self.at_state(phase, reset_free_energies=False)
+        else:
+            self._set_phase_ref(phase_ref, self._Tm, self._Tb)
         self._init_reactions(self._Hf, self._S0, self._LHV, self._HHV, 
                              self._combustion, self.atoms)
+        if free_energies:
+            self._init_energies(self._Cn, self._Hvap, self._Psat, self._Hfus, self._Sfus,
+                                self._Tm, self._Tb, self._eos, self._phase_ref, self._S0)
         if self._formula and self._Hf is not None: self.reset_combustion_data()
         TDependentProperty.RAISE_PROPERTY_CALCULATION_ERROR = True
         return self
 
     @property
     def charge(self):
-        """Charge of chemical as described in the chemcial formula."""
+        """Charge of chemical as described in the chemical formula."""
         return charge_from_formula(self.formula)
 
     def copy(self, ID, CAS=None, **data):
@@ -685,24 +751,12 @@ class Chemical:
         new._aliases = set()
         TDependentProperty.RAISE_PROPERTY_CALCULATION_ERROR = False
         new._init_energies(new.Cn, new.Hvap, new.Psat, new.Hfus, new.Sfus, new.Tm,
-                           new.Tb, new.eos, new.phase_ref)
+                           new.Tb, new.eos, new.phase_ref, new.S0)
         TDependentProperty.RAISE_PROPERTY_CALCULATION_ERROR = True
-        new._label_handles()
         for i,j in data.items(): setfield(new, i , j)
         return new
     __copy__ = copy
 
-    def _label_handles(self):
-        handles = (self._Psat, self._Hvap, self._sigma, self._epsilon,
-                   self._V, self._Cn, self._mu, self._kappa)
-        isa = isinstance
-        label = self.CAS
-        for handle in handles:
-            if isa(handle, PhaseHandle):
-                for i, j in handle: j.CASRN = label
-            else:
-                handle.CASRN = label
-
     def __reduce__(self):
         return unpickle_chemical, (get_chemical_data(self),)
     
@@ -874,11 +928,14 @@ class Chemical:
     
     @property
     def iupac_name(self):
-        """[str] Standard name as defined by IUPAC."""
+        """[tuple[str]] Standard names as defined by IUPAC."""
         return self._iupac_name
     @iupac_name.setter
     def iupac_name(self, iupac_name):
-        self._iupac_name = str(iupac_name)
+        if isinstance(iupac_name, str):
+            self._iupac_name = (iupac_name,)
+        else:
+            self._iupac_name = iupac_name
     
     @property
     def pubchemid(self):
@@ -1015,7 +1072,7 @@ class Chemical:
         
     @property
     def epsilon(self): 
-        """Relative permitivity [-]."""
+        """Relative permittivity [-]."""
         return self._epsilon
     @epsilon.setter
     def epsilon(self, value):
@@ -1140,7 +1197,7 @@ class Chemical:
     
     @property
     def omega(self):
-        """Accentric factor [-]."""
+        """Acentric factor [-]."""
         return self._omega
     @omega.setter
     def omega(self, omega):
@@ -1324,7 +1381,9 @@ class Chemical:
               Tt=None, Pt=None, Hf=None, S0=None, LHV=None, combustion=None,
               HHV=None, Hfus=None, dipole=None,
               similarity_variable=None, iscyclic_aliphatic=None, aliases=None,
-              synonyms=None, *, metadata=None, phase=None):
+              synonyms=None, *, metadata=None, phase=None,
+              free_energies=True,
+        ):
         """
         Reset all chemical properties.
 
@@ -1381,16 +1440,34 @@ class Chemical:
                            self._iscyclic_aliphatic, self._eos, self.has_hydroxyl,
                            self._Hfus)
         self._locked_state = None
-        if phase: self.at_state(phase)
         self._estimate_missing_properties(self._Psat)
-        self._init_energies(self._Cn, self._Hvap, self._Psat, self._Hfus, self._Sfus,
-                            self._Tm, self._Tb, self._eos, phase_ref)
+        if phase:
+            self.at_state(phase, reset_free_energies=False)
+        else:
+            self._set_phase_ref(phase_ref, self._Tm, self._Tb)
         self._init_reactions(Hf, S0, LHV, HHV, combustion, atoms)
+        if free_energies:
+            self._init_energies(self._Cn, self._Hvap, self._Psat, self._Hfus, self._Sfus,
+                                self._Tm, self._Tb, self._eos, self._phase_ref, self._S0)
         if self._formula and self._Hf is not None: self.reset_combustion_data()
         TDependentProperty.RAISE_PROPERTY_CALCULATION_ERROR = True
 
+    def _set_phase_ref(self, phase_ref, Tm, Tb):
+        if phase_ref:
+            # "Liquid", "Solid", and "Gas" are also valid arguments
+            phase_ref = phase_ref[0].lower()
+        else:
+            T_ref = self.T_ref
+            if Tm and T_ref <= Tm:
+                phase_ref = 's'
+            elif Tb and T_ref >= Tb:
+                phase_ref = 'g'
+            else:
+                phase_ref = 'l'
+        self._phase_ref = phase_ref
+
     def reset_combustion_data(self, method="Stoichiometry"):
-        """Reset combustion data (LHV, HHV, and combution attributes)
+        """Reset combustion data (LHV, HHV, and combustion attributes)
         based on the molecular formula and the heat of formation."""
         cd = combustion_data(self.atoms, Hf=self._Hf, MW=self._MW, method=method)
         if not self._MW: self._MW = cd.MW
@@ -1404,7 +1481,7 @@ class Chemical:
             self._eos = create_eos(self.EOS_default, self._Tc, self._Pc, self._omega)
         TDependentProperty.RAISE_PROPERTY_CALCULATION_ERROR = False
         self._init_energies(self._Cn, self._Hvap, self._Psat, self._Hfus, self._Sfus,
-                            self._Tm, self._Tb, self._eos, self._phase_ref)
+                            self._Tm, self._Tb, self._eos, self._phase_ref, self._S0)
         TDependentProperty.RAISE_PROPERTY_CALCULATION_ERROR = True
 
     ### Initializers ###
@@ -1483,7 +1560,7 @@ class Chemical:
             S0 = absolute_entropy_of_formation(self._CAS, self._phase_ref,
                                                Svap_298K, self.Sfus) or 0.
         self._Hf = Hf
-        self.S0 = S0
+        self._S0 = S0
         atoms = atoms or self.atoms
         if not all([LHV, HHV, combustion]) and atoms and Hf:
             cd = combustion_data(atoms, Hf=self._Hf, MW=self._MW, missing_handling='Ash')
@@ -1511,7 +1588,7 @@ class Chemical:
         Vs = VolumeSolid(CAS, MW, Tt, Vml_Tm)
         Vg = VolumeGas(MW=MW, Tc=Tc, Pc=Pc, omega=omega, dipole=dipole,
                        eos=[eos], CASRN=CAS)
-        self._V = V = PhaseTPHandle('V', Vs, Vl, Vg)
+        self._V = V = PhaseTPHandle('V', Vs, Vl, Vg, Tc)
         if CAS in sugar_solid_densities: Vs.add_method(sugar_solid_densities[CAS])
         
         # Sensible heats
@@ -1520,7 +1597,7 @@ class Chemical:
                               iscyclic_aliphatic=iscyclic_aliphatic)
         Cnl = HeatCapacityLiquid(CASRN=CAS, Tc=Tc, omega=omega, MW=MW, 
                                  similarity_variable=similarity_variable, Cpgm=Cng)
-        self._Cn = Cn = PhaseTHandle('Cn', Cns, Cnl, Cng)
+        self._Cn = Cn = PhaseTHandle('Cn', Cns, Cnl, Cng, Tc)
         
         # Latent heats
         Zl = CompressibilityFactor(Vl)
@@ -1541,7 +1618,7 @@ class Chemical:
                               omega=omega, Psat=Psat, Vml=V.l)
         mug = ViscosityGas(CASRN=CAS, MW=MW, Tc=Tc, Pc=Pc, Zc=Zc, dipole=dipole,
                            Vmg=Vg)
-        self._mu = mu = PhaseTPHandle('mu', mul, mul, mug)
+        self._mu = mu = PhaseTPHandle('mu', mul, mul, mug, Tc)
         
         # Conductivity
         kappal = ThermalConductivityLiquid(CASRN=CAS, MW=MW, Tm=Tm, Tb=Tb, 
@@ -1549,7 +1626,7 @@ class Chemical:
         kappag = ThermalConductivityGas(CASRN=CAS, MW=MW, Tb=Tb, Tc=Tc, Pc=Pc, 
                                         Vc=Vc, Zc=Zc, omega=omega, dipole=dipole, 
                                         Vmg=V.g, Cpgm=Cn.g, mug=mu.g)
-        self._kappa = PhaseTPHandle('kappa', kappal, kappal, kappag)
+        self._kappa = PhaseTPHandle('kappa', kappal, kappal, kappag, Tc)
         
         # Surface tension
         Hvap_Tb = self._Hvap(Tb) if Tb else None
@@ -1561,7 +1638,6 @@ class Chemical:
         
         # Other
         self._epsilon = PermittivityLiquid(CAS)
-        self._label_handles()
         
 
     def _estimate_missing_properties(self, Psat):
@@ -1601,13 +1677,12 @@ class Chemical:
                 omega = acentric_factor_LK(Tb, Tc, Pc)
             self._omega = omega
 
-    def _init_energies(self, Cn, Hvap, Psat, Hfus, Sfus, Tm, Tb, eos, 
-                       phase_ref=None):        
+    def _init_energies(self, Cn, Hvap, Psat, Hfus, Sfus, Tm, Tb, eos, phase_ref, S0):        
         # Reference
         P_ref = self.P_ref
         T_ref = self.T_ref
         H_ref = self.H_ref
-        S0 = 0. # Replaced later in _init_reactions method
+        Tc = self._Tc
         single_phase = self._locked_state
         if isinstance(Cn, PhaseHandle):
             Cn_s = Cn.s
@@ -1616,8 +1691,7 @@ class Chemical:
             has_Cns = bool(Cn_s)
             has_Cnl = bool(Cn_l)
             has_Cng = bool(Cn_g)
-        elif Cn and single_phase:
-            self._phase_ref = single_phase
+        elif single_phase:
             self._H = Enthalpy.functor(Cn, T_ref, H_ref)
             if single_phase == 'g':
                 self._S = EntropyGas.functor(Cn, T_ref, P_ref, S0)
@@ -1626,23 +1700,14 @@ class Chemical:
             Cn_s = Cn_l = Cn_g = Cn
             has_Cns = has_Cnl = has_Cng = False
             if single_phase == 'l':
-                has_Cnl = True
+                has_Cnl = bool(Cn_l)
             elif single_phase == 's':
-                has_Cns = True
+                has_Cns = bool(Cn_s)
             elif single_phase == 'g':
-                has_Cng = True    
+                has_Cng = bool(Cn_g)  
         else:
             has_Cns = has_Cnl = has_Cng = False
-        if phase_ref: phase_ref = phase_ref[0]
-        self._phase_ref = phase_ref
         if any((has_Cns, has_Cnl, has_Cng)):
-            if not phase_ref:
-                if Tm and T_ref <= Tm:
-                    self._phase_ref = phase_ref = 's'
-                elif Tb and T_ref >= Tb:
-                    self._phase_ref = phase_ref = 'g'
-                else:
-                    self._phase_ref = phase_ref = 'l'
             if Hvap:
                 Hvap_Tb = Hvap(Tb) if Tb else None
                 Svap_Tb = Hvap_Tb / Tb if Hvap_Tb else None
@@ -1718,56 +1783,65 @@ class Chemical:
                     H_dep_Tb_Pb_g = 0.
             
             # Enthalpy and Entropy
-            if not single_phase:
+            if single_phase:
+                # Reference state does not matter because phase will not change
+                self._H = Enthalpy.functor(Cn, T_ref, H_ref)
+                self._S = Entropy.functor(Cn, T_ref, S0)
+                # Excess energies
+                if single_phase == 's':
+                    self._H_excess = Excess_Solid_Enthalpy_Ref_Solid.functor()
+                    self._S_excess = Excess_Solid_Entropy_Ref_Solid.functor()
+                elif single_phase == 'l':
+                    self._H_excess = Excess_Liquid_Enthalpy_Ref_Liquid.functor()
+                    self._S_excess = Excess_Liquid_Entropy_Ref_Liquid.functor()
+                elif single_phase == 'g':
+                    self._H_excess = Excess_Gas_Enthalpy_Ref_Gas.functor(eos, H_dep_ref_g)
+                    self._S_excess = Excess_Gas_Entropy_Ref_Gas.functor(eos, S_dep_ref_g)
+            else:
                 if phase_ref == 's':
                     sdata = (Cn_s, T_ref, H_ref)
                     ldata = (Cn_l, H_int_T_ref_to_Tm_s, Hfus, Tm, H_ref)
                     gdata = (Cn_g, H_int_T_ref_to_Tm_s, Hfus, H_int_Tm_to_Tb_l, Hvap_Tb, Tb, H_ref)
-                    self._H = EnthalpyRefSolid(sdata, ldata, gdata)
+                    self._H = EnthalpyRefSolid(sdata, ldata, gdata, Tc)
                     sdata = (Cn_s, T_ref, S0)
                     ldata = (Cn_l, S_int_T_ref_to_Tm_s, Sfus, Tm, S0)
                     gdata = (Cn_g, S_int_T_ref_to_Tm_s, Sfus, S_int_Tm_to_Tb_l, Svap_Tb, Tb, P_ref, S0)
-                    self._S = EntropyRefSolid(sdata, ldata, gdata)
+                    self._S = EntropyRefSolid(sdata, ldata, gdata, Tc)
                 elif phase_ref == 'l':
                     sdata = (Cn_s, H_int_Tm_to_T_ref_l, Hfus, Tm, H_ref)
                     ldata = (Cn_l, T_ref, H_ref)
                     gdata = (Cn_g, H_int_T_ref_to_Tb_l, Hvap_Tb, Tb, H_ref)
-                    self._H = EnthalpyRefLiquid(sdata, ldata, gdata)
+                    self._H = EnthalpyRefLiquid(sdata, ldata, gdata, Tc)
                     sdata = (Cn_s, S_int_Tm_to_T_ref_l, Sfus, Tm, S0)
                     ldata = (Cn_l, T_ref, S0)
                     gdata = (Cn_g, S_int_T_ref_to_Tb_l, Svap_Tb, Tb, P_ref, S0)
-                    self._S = EntropyRefLiquid(sdata, ldata, gdata)
+                    self._S = EntropyRefLiquid(sdata, ldata, gdata, Tc)
                 elif phase_ref == 'g':
                     sdata = (Cn_s, H_int_Tb_to_T_ref_g, Hvap_Tb, H_int_Tm_to_Tb_l, Hfus, Tm, H_ref)
                     ldata = (Cn_l, H_int_Tb_to_T_ref_g, Hvap_Tb, Tb, H_ref)
                     gdata = (Cn_g, T_ref, H_ref)
-                    self._H = EnthalpyRefGas(sdata, ldata, gdata)
+                    self._H = EnthalpyRefGas(sdata, ldata, gdata, Tc)
                     sdata = (Cn_s, S_int_Tb_to_T_ref_g, Svap_Tb, S_int_Tm_to_Tb_l, Sfus, Tm, S0)
                     ldata = (Cn_l, S_int_Tb_to_T_ref_g, Svap_Tb, Tb, S0)
                     gdata = (Cn_g, T_ref, P_ref, S0)
-                    self._S = EntropyRefGas(sdata, ldata, gdata)
+                    self._S = EntropyRefGas(sdata, ldata, gdata, Tc)
             
-            # Excess energies
-            if phase_ref == 's':
-                self._H_excess = ExcessEnthalpyRefSolid((), (), ())
-                self._S_excess = ExcessEntropyRefSolid((), (), ())
-            elif phase_ref == 'l':
-                gdata = (eos, H_dep_T_ref_Pb, H_dep_ref_l, H_dep_Tb_Pb_g)
-                self._H_excess = ExcessEnthalpyRefLiquid((), (), gdata)
-                gdata = (eos, S_dep_T_ref_Pb, S_dep_ref_l, S_dep_Tb_Pb_g)
-                self._S_excess = ExcessEntropyRefLiquid((), (), gdata)
-            elif phase_ref == 'g':
-                ldata = (eos, H_dep_Tb_Pb_g, H_dep_Tb_P_ref_g)
-                gdata = (eos, H_dep_ref_g)
-                self._H_excess = ExcessEnthalpyRefGas((), ldata, gdata)
-                ldata = (eos, S_dep_Tb_Pb_g, S_dep_Tb_P_ref_g)
-                gdata = (eos, S_dep_ref_g)
-                self._S_excess = ExcessEntropyRefGas((), ldata, gdata)
-                
-            if single_phase:
-                getfield = getattr
-                self._H_excess = getfield(self._H_excess, single_phase)
-                self._S_excess = getfield(self._S_excess, single_phase)
+                # Excess energies
+                if phase_ref == 's':
+                    self._H_excess = ExcessEnthalpyRefSolid((), (), (), Tc)
+                    self._S_excess = ExcessEntropyRefSolid((), (), (), Tc)
+                elif phase_ref == 'l':
+                    gdata = (eos, H_dep_T_ref_Pb, H_dep_ref_l, H_dep_Tb_Pb_g)
+                    self._H_excess = ExcessEnthalpyRefLiquid((), (), gdata, Tc)
+                    gdata = (eos, S_dep_T_ref_Pb, S_dep_ref_l, S_dep_Tb_Pb_g)
+                    self._S_excess = ExcessEntropyRefLiquid((), (), gdata, Tc)
+                elif phase_ref == 'g':
+                    ldata = (eos, H_dep_Tb_Pb_g, H_dep_Tb_P_ref_g)
+                    gdata = (eos, H_dep_ref_g)
+                    self._H_excess = ExcessEnthalpyRefGas((), ldata, gdata, Tc)
+                    ldata = (eos, S_dep_Tb_Pb_g, S_dep_Tb_P_ref_g)
+                    gdata = (eos, S_dep_ref_g)
+                    self._S_excess = ExcessEntropyRefGas((), ldata, gdata, Tc)
         else:
             self._H = self._S = self._S_excess = self._H_excess = None
             
@@ -1909,7 +1983,7 @@ class Chemical:
                 Cn_phase = getfield(Cn, phase_ref)
                 Cn_phase.add_method(4.18*MW)
             self.reset_free_energies()
-        if not self._H:
+        if not getattr(self, '_H', None):
             self.reset_free_energies()
         missing = set(properties)
         missing.difference_update({'MW', 'CAS', 'Cn', 'Hf', 'sigma',
@@ -1962,7 +2036,7 @@ class Chemical:
         
         """
         getfield = getattr
-        return [i for i in (properties or _checked_properties) if not getfield(self, i)]
+        return [i for i in (properties or _checked_properties) if not getfield(self, i, None)]
     
     def copy_models_from(self, other, names=None):
         """Copy models from other."""
@@ -2022,7 +2096,7 @@ class Chemical:
     def N_solutes(self, N_solutes):
         self._N_solutes = int(N_solutes)
     
-    def at_state(self, phase, copy=False):
+    def at_state(self, phase, copy=False, reset_free_energies=True):
         """
         Set the state of chemical.
         
@@ -2031,23 +2105,27 @@ class Chemical:
         >>> from thermosteam import Chemical
         >>> N2 = Chemical('N2')
         >>> N2.at_state(phase='g')
-        >>> N2.H(298.15) # No longer a function of phase
+        >>> N2.H(298.15, None) # No longer a function of phase
         0.0
         """
         if copy:
             new = self.copy(self._ID)
             new.at_state(phase)
             return new
+        if phase: 
+            phase = phase[0].lower()
+        else:
+            raise ValueError(f"invalid phase {repr(phase)}")
         locked_state = self._locked_state
         if locked_state:
             if locked_state != phase:
                 raise TypeError(f"{self}'s state is already locked")   
             else:
                 return         
-        elif phase:
-            lock_phase(self, phase)
         else:
-            raise ValueError(f"invalid phase {repr(phase)}")
+            lock_phase(self, phase)
+        if reset_free_energies:
+            self.reset_free_energies()
     
     def show(self):
         """Print all specifications"""
@@ -2084,7 +2162,6 @@ class Chemical:
         return f"Chemical('{self}')"
     
 def lock_phase(chemical, phase):
-    phase = phase[0]
     check_phase(phase)
     getfield = getattr
     setfield = object.__setattr__
@@ -2100,11 +2177,8 @@ def lock_phase(chemical, phase):
         if hasfield(phase_property, phase):
             functor = getfield(phase_property, phase)
             setfield(chemical, field, functor)
-    Cn = chemical.Cn
+    # Reference state does not matter because phase will not change
     chemical._phase_ref = phase
-    chemical._H = Enthalpy.functor(Cn, chemical.T_ref, chemical.H_ref)
-    S0 = chemical._S0 if hasfield(chemical, '_S0') else 0.
-    chemical._S = Entropy.functor(Cn, chemical.T_ref, S0)
     chemical._locked_state = phase
 
 # # Fire Safety Limits
@@ -2133,4 +2207,4 @@ def lock_phase(chemical, phase):
 # # Analytical
 # self.RI, self.RIT = refractive_index(CASRN=CAS)
 # self.conductivity, self.conductivityT = conductivity(CASRN=CAS)
-    
+    
\ No newline at end of file
