@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # BioSTEAM: The Biorefinery Simulation and Techno-Economic Analysis Modules
-# Copyright (C) 2020-2021, Yoel Cortes-Pena <yoelcortes@gmail.com>
+# Copyright (C) 2020-2023, Yoel Cortes-Pena <yoelcortes@gmail.com>
 # 
 # This module is under the UIUC open-source license. See 
 # github.com/BioSTEAMDevelopmentGroup/biosteam/blob/master/LICENSE.txt
@@ -48,7 +48,7 @@ class Chemicals:
            * SMILES (prefix with 'SMILES=' to ensure smiles parsing)
            * CAS number
     cache : bool, optional
-        Wheather or not to use cached chemicals.
+        Whether or not to use cached chemicals.
     
     Examples
     --------
@@ -115,7 +115,7 @@ class Chemicals:
     UndefinedChemicalAlias: 'Butane'
     
     """
-    def __new__(cls, chemicals, cache=False):
+    def __new__(cls, chemicals, cache=None):
         self = super().__new__(cls)
         isa = isinstance
         setfield = setattr
@@ -386,11 +386,11 @@ class CompiledChemicals(Chemicals):
         >>> chemicals = tmo.CompiledChemicals(['Water', 'Methanol', 'Ethanol'], cache=True)
         >>> chemicals.define_group('Alcohol', ['Methanol', 'Ethanol'], composition=[0.5, 0.5])
         >>> chemicals.get_index('Alcohol')
-        [1, 2]
+        array([1, 2])
         >>> chemicals.get_index(('Water', 'Alcohol'))
-        [0, [1, 2]]
+        [0, array([1, 2])]
         
-        By defining a chemical group, you can conviniently use indexers
+        By defining a chemical group, you can conveniently use indexers
         to retrieve the total value of the group:
             
         >>> # Single phase stream case
@@ -471,6 +471,8 @@ class CompiledChemicals(Chemicals):
             if i in self._group_wt_compositions:
                 raise ValueError(f"'{i}' is a group; cannot define new group using other groups")
         index = self.indices(IDs)
+        self.__dict__[name] = [self.tuple[i] for i in index]
+        self._index[name] = np.array(index, dtype=int)
         composition = np.asarray(composition, float)
         if wt:
             composition_wt = composition
@@ -480,8 +482,11 @@ class CompiledChemicals(Chemicals):
             composition_mol = composition
         self._group_wt_compositions[name] = composition_wt / composition_wt.sum()
         self._group_mol_compositions[name] = composition_mol / composition_mol.sum()
-        self._index[name] = index
-        self.__dict__[name] = [self.tuple[i] for i in index]
+    
+    @property
+    def chemical_groups(self) -> frozenset[str]:
+        """All defined chemical groups."""
+        return frozenset(self._group_mol_compositions)
     
     def refresh_constants(self):
         """
@@ -552,8 +557,11 @@ class CompiledChemicals(Chemicals):
         repeated_names = set()
         names = set()
         all_names_list = []
+        isa = isinstance
         for i in chemicals:
             if not i.iupac_name: i.iupac_name = ()
+            elif isa(i.iupac_name, str):
+                i.iupac_name = (i.iupac_name,)
             all_names = set([*i.iupac_name, *i.aliases, i.common_name, i.formula])
             all_names_list.append(all_names)
             for name in all_names:
@@ -672,7 +680,8 @@ class CompiledChemicals(Chemicals):
         
         """
         k = self._index[ID]
-        return [i for i, j in self._index.items() if j==k] 
+        isa = isinstance
+        return [i for i, j in self._index.items() if isa(j, int) and (j==k)]
 
     get_synonyms = get_aliases
 
@@ -731,7 +740,7 @@ class CompiledChemicals(Chemicals):
     
     def ones(self):
         """
-        Return an array of ones with entries that correspond to the orded chemical IDs.
+        Return an array of ones with entries that correspond to the ordered chemical IDs.
         
         Examples
         --------
@@ -745,7 +754,7 @@ class CompiledChemicals(Chemicals):
     
     def kwarray(self, ID_data):
         """
-        Return an array with entries that correspond to the orded chemical IDs.
+        Return an array with entries that correspond to the ordered chemical IDs.
         
         Parameters
         ----------
@@ -962,7 +971,7 @@ class CompiledChemicals(Chemicals):
         Parameters
         ----------
         IDs : iterable
-              Chemical indentifiers.
+              Chemical identifiers.
 
         Examples
         --------
@@ -1069,18 +1078,24 @@ class CompiledChemicals(Chemicals):
         except KeyError:
             isa = isinstance
             kind = 0 # [int] Kind of index: 0 - normal, 1 - chemical group, 2 - nested chemical group
+            ndarray = np.ndarray
             if isa(key, str):
                 index = self.index(key)
-                if isa(index, list): kind = 1 
+                if isa(index, ndarray): kind = 1 
             elif isa(key, tuple):
                 index = self.indices(key)
                 for i in index:
-                    if isa(i, list): kind = 2
+                    if isa(i, ndarray): 
+                        kind = 2
+                        break
+                else:
+                    index = np.array(index, dtype=int)
             elif key is ...:
                 index = slice(None)
             else: # pragma: no cover
                 raise TypeError("only strings, sequences of strings, and ellipsis are valid index keys")
             index_cache[key] = index, kind
+            if len(index_cache) > 1000: index_cache.pop(index_cache.__iter__().__next__())
         except TypeError:
             raise TypeError("only strings, sequences of strings, and ellipsis are valid index keys")
         return index, kind
@@ -1110,10 +1125,10 @@ class CompiledChemicals(Chemicals):
         >>> chemicals = CompiledChemicals(['Water', 'Methanol', 'Ethanol'])
         >>> data = chemicals.kwarray(dict(Water=2., Ethanol=1.))
         >>> chemicals.get_vle_indices(data!=0)
-        [0, 2]
+        array([0, 2])
         
         """
-        return [i for i, j in enumerate(self._has_vle & nonzeros) if j]
+        return np.array([i for i, j in enumerate(self._has_vle & nonzeros) if j], int)
     
     def get_lle_indices(self, nonzeros):
         """
@@ -1126,10 +1141,10 @@ class CompiledChemicals(Chemicals):
         >>> chemicals = CompiledChemicals(['Water', 'Methanol', 'Ethanol'])
         >>> data = chemicals.kwarray(dict(Water=2., Ethanol=1.))
         >>> chemicals.get_lle_indices(data!=0)
-        [0, 2]
+        array([0, 2])
         
         """
-        return [i for i, j in enumerate(self._has_lle & nonzeros) if j]
+        return np.array([i for i, j in enumerate(self._has_lle & nonzeros) if j], int)
     
     def __repr__(self):
         return f"{type(self).__name__}([{', '.join(self.IDs)}])"
