@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # BioSTEAM: The Biorefinery Simulation and Techno-Economic Analysis Modules
-# Copyright (C) 2020-2021, Yoel Cortes-Pena <yoelcortes@gmail.com>
+# Copyright (C) 2020-2023, Yoel Cortes-Pena <yoelcortes@gmail.com>
 # 
 # This module is under the UIUC open-source license. See 
 # github.com/BioSTEAMDevelopmentGroup/biosteam/blob/master/LICENSE.txt
@@ -15,6 +15,7 @@ from .indexer import MolarFlowIndexer
 from ._phase import phase_tuple
 from . import equilibrium as eq
 from . import utils
+from .indexer import nonzeros
 import numpy as np
 from typing import TYPE_CHECKING
 if TYPE_CHECKING:
@@ -264,8 +265,8 @@ class MultiStream(Stream):
         >>> s1.show('cwt')
         MultiStream: s1
          phases: ('g', 'l', 's'), T: 298.15 K, P: 101325 Pa
-         composition: (g) Ethanol  1
-                          -------  2 kg/hr
+         composition (%): (g) Ethanol  100
+                              ---------  2 kg/hr
         
         """
         imol = self._imol
@@ -308,21 +309,24 @@ class MultiStream(Stream):
             else:
                 literal = (imol._phases, thermal_condition._T, thermal_condition._P)
             last_literal, last_composition = self._property_cache_key
-            if name in property_cache and literal == last_literal and (composition == last_composition).all():
-                value = property_cache.get(name)
+            if literal == last_literal and (composition == last_composition).all():
+                if name in property_cache:
+                    value = property_cache.get(name)
+                    return value * total if flow else value
             else:
-                self._property_cache_key = (literal, composition)
-                if nophase:
-                    calculate = getattr(self.mixture, name)
-                    self._property_cache[name] = value = calculate(
-                        composition.sum(axis=0), *self.thermal_condition
-                    )
-                else:
-                    calculate = getattr(self.mixture, 'x' + name)
-                    self._property_cache[name] = value = calculate(
-                        zip(imol._phases, composition), *self.thermal_condition
-                    )
-        return value * total if flow else value
+                property_cache.clear()
+            self._property_cache_key = (literal, composition)
+            if nophase:
+                calculate = getattr(self.mixture, name)
+                self._property_cache[name] = value = calculate(
+                    composition.sum(axis=0), *self.thermal_condition
+                )
+            else:
+                calculate = getattr(self.mixture, 'x' + name)
+                self._property_cache[name] = value = calculate(
+                    zip(imol._phases, composition), *self.thermal_condition
+                )
+            return value * total if flow else value
     
     def reset_cache(self):
         """Reset cache regarding equilibrium methods."""
@@ -541,7 +545,8 @@ class MultiStream(Stream):
     @property
     def vapor_fraction(self) -> float:
         """Molar vapor fraction."""
-        return get_phase_fraction(self, 'gG')
+        return self.imol['g'].sum() / F_mol if 'g' in self.phases and (F_mol:=self.F_mol) != 0. else 0.
+        
     @property
     def liquid_fraction(self) -> float:
         """Molar liquid fraction."""
@@ -598,6 +603,49 @@ class MultiStream(Stream):
             tc1._T = tc2._T = tc._T
             tc1._P = tc2._P = tc._P
     
+    def copy_like(self, other):
+        """
+        Copy all conditions of another stream.
+
+        Examples
+        --------
+        Copy data from another stream with the same property package:
+        
+        >>> import thermosteam as tmo
+        >>> tmo.settings.set_thermo(['Water', 'Ethanol'], cache=True)
+        >>> s1 = tmo.MultiStream('s1', l=[('Water', 20), ('Ethanol', 10)], units='kg/hr')
+        >>> s2 = tmo.Stream('s2', Water=2, units='kg/hr')
+        >>> s1.copy_like(s2)
+        >>> s1.show(flow='kg/hr')
+        MultiStream: s1
+         phases: ('g', 'l'), T: 298.15 K, P: 101325 Pa
+         flow (kg/hr): (l) Water  2
+         
+        Copy data from another stream with a different property package:
+        
+        >>> import thermosteam as tmo
+        >>> tmo.settings.set_thermo(['Water', 'Ethanol'], cache=True)
+        >>> s1 = tmo.MultiStream('s1', l=[('Water', 20), ('Ethanol', 10)], units='kg/hr')
+        >>> tmo.settings.set_thermo(['Water'], cache=True)
+        >>> s2 = tmo.Stream('s2', Water=2, units='kg/hr')
+        >>> s1.copy_like(s2)
+        >>> s1.show(flow='kg/hr')
+        MultiStream: s1
+         phases: ('g', 'l'), T: 298.15 K, P: 101325 Pa
+         flow (kg/hr): (l) Water  2
+
+        """
+        if isinstance(other, MultiStream):
+            self.phases = other.phases
+            imol = other._imol
+        elif other.phase not in self.phases:
+            self.phases = (other.phase, *self.phases)
+            imol = other._imol
+        else:
+            imol = other._imol
+        self._imol.copy_like(imol)
+        self._thermal_condition.copy_like(other._thermal_condition)
+    
     def copy_flow(self, 
                   other: Stream, 
                   phase: Optional[Sequence[str]|str]=...,
@@ -982,33 +1030,20 @@ class MultiStream(Stream):
     
     ### Representation ###
     
-    def _info(self, layout, T, P, flow, composition, N, IDs):
+    def _info_str(self, T_units, P_units, flow_units, composition, N_max, all_IDs, indexer, factor):
         """Return string with all specifications."""
-        flow, composition, N = self._translate_layout(layout, flow, composition, N)
-        from .indexer import nonzeros
-        if not IDs: IDs = self.chemicals.IDs
         basic_info = self._basic_info()
-        all_IDs, _ = nonzeros(IDs, self.imol[IDs])
-        all_IDs = tuple(all_IDs)
-        display_units = self.display_units
-        T_units = T or display_units.T
-        P_units = P or display_units.P
-        flow_units = flow or display_units.flow
-        N_max = display_units.N if N is None else N
-        if composition is None: composition = display_units.composition
         basic_info += Stream._info_phaseTP(self, self.phases, T_units, P_units)
         N_all_IDs = len(all_IDs)
         if N_all_IDs == 0:
             return basic_info + ' flow: 0' 
 
         # Length of chemical column
-        all_lengths = [len(i) for i in all_IDs]
-        maxlen = max(all_lengths) 
-
-        name, factor = self._get_flow_name_and_factor(flow_units)
-        indexer = getattr(self, 'i' + name)
+        all_lengths = [len(i) for i in all_IDs[:N_max]]
+        maxlen = max(all_lengths) + 2
+        
         if composition:
-            first_line = " composition:"
+            first_line = " composition (%):"
         else:
             first_line = f' flow ({flow_units}):'
         first_line_spaces = len(first_line)*" "
@@ -1021,7 +1056,7 @@ class MultiStream(Stream):
             if not IDs: continue
             if composition:
                 total_flow = data.sum()
-                data = data/total_flow
+                data = 100 * data/total_flow
         
             # Get basic structure for phase data            
             beginning = (first_line or first_line_spaces) + f' ({phase}) '
@@ -1037,9 +1072,10 @@ class MultiStream(Stream):
             for i in range(N):
                 spaces = ' ' * (maxlen - lengths[i])
                 if i: flow_rates += new_line    
-                flow_rates += f'{IDs[i]} ' + spaces + f' {data[i]:.3g}'
+                flow_rates += f'{IDs[i]}' + spaces + f'{data[i]:.3g}'
             if too_many_chemicals:
-                flow_rates += new_line + '...'
+                spaces = ' ' * (maxlen - 3)
+                flow_rates += new_line + '... ' + spaces + f' {data[N_max:].sum():.3g}'
             if composition:
                 dashes = '-' * maxlen
                 flow_rates += f"{new_line}{dashes}  {total_flow:.3g} {flow_units}"
