@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # BioSTEAM: The Biorefinery Simulation and Techno-Economic Analysis Modules
-# Copyright (C) 2020-2021, Yoel Cortes-Pena <yoelcortes@gmail.com>
+# Copyright (C) 2020-2023, Yoel Cortes-Pena <yoelcortes@gmail.com>
 # 
 # This module is under the UIUC open-source license. See 
 # github.com/BioSTEAMDevelopmentGroup/biosteam/blob/master/LICENSE.txt
@@ -10,10 +10,16 @@
 from .exceptions import UndefinedPhase
 
 __all__ = ('Phase', 'LockedPhase', 'NoPhase', 'PhaseIndexer',
-           'phase_tuple')
+           'phase_tuple', 'valid_phases')
 
 new = object.__new__
-valid_phases = {'s', 'l', 'g', 'S', 'L'}
+valid_phases = {
+    's': 'solid',
+    'l': 'liquid',
+    'g': 'gas',
+    'S': 'SOLID',
+    'L': 'LIQUID',
+}
 
 def check_phase(phase):
     """
@@ -140,7 +146,7 @@ class Phase:
         self._phase = phase
     
     def copy(self):
-        return self.__class__(self.phase)
+        return Phase(self.phase)
     __copy__ = copy
     
     def __repr__(self):
