@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # BioSTEAM: The Biorefinery Simulation and Techno-Economic Analysis Modules
-# Copyright (C) 2020-2021, Yoel Cortes-Pena <yoelcortes@gmail.com>
+# Copyright (C) 2020-2023, Yoel Cortes-Pena <yoelcortes@gmail.com>
 # 
 # This module is under the UIUC open-source license. See 
 # github.com/BioSTEAMDevelopmentGroup/biosteam/blob/master/LICENSE.txt
@@ -11,11 +11,19 @@ from __future__ import annotations
 import thermosteam as tmo
 from typing import Optional, Iterable
 from .units_of_measure import AbsoluteUnitsOfMeasure
-try: import biosteam as bst
-except: pass
+from typing import TYPE_CHECKING
+if TYPE_CHECKING:
+    from .equilibrium import (
+        ActivityCoefficients,
+        FugacityCoefficients,
+        PoyintingCorrectionFactors,
+    )
+    from .mixture import Mixture
 
 __all__ = ('settings', 'ProcessSettings')
 
+# %%
+
 def raise_no_thermo_error():
     raise RuntimeError("no available 'Thermo' object; "
                        "use settings.set_thermo to set the default thermo object")
@@ -24,6 +32,8 @@ def raise_no_flashpkg_error():
     raise RuntimeError("no available 'FlashPackage' object; "
                        "use settings.set_flashpkg to set the default flash package object")
     
+# %%
+    
 class ProcessSettings:
     """
     A compilation of all settings that may affect BioSTEAM results, including
@@ -64,7 +74,7 @@ class ProcessSettings:
         ),
         Gamma=DortmundActivityCoefficients,
         Phi=IdealFugacityCoefficients,
-        PCF=IdealPoyintingCorrectionFactors
+        PCF=MockPoyintingCorrectionFactors
     )
 
     Access defined chemicals:
@@ -95,69 +105,15 @@ class ProcessSettings:
     def __new__(cls):
         return settings
     
-    @property
-    def CEPCI(self) -> float:
-        """Chemical engineering plant cost index (defaults to 567.5 at 2017)."""
-        return bst.CE
-    @CEPCI.setter
-    def CEPCI(self, CEPCI):
-        bst.CE = CEPCI
-    
-    @property
-    def utility_characterization_factors(self) ->  dict[tuple[str, str], tuple[float, AbsoluteUnitsOfMeasure]]:
-        """Utility characterization factor data (value and units) by agent ID 
-        and impact key."""
-        return bst.HeatUtility.characterization_factors
-    @utility_characterization_factors.setter
-    def utility_characterization_factors(self, utility_characterization_factors):
-        bst.HeatUtility.characterization_factors = utility_characterization_factors
-    
-    @property
-    def cooling_agents(self) -> list[bst.UtilityAgent]:
-        """All cooling utilities available."""
-        return bst.HeatUtility.cooling_agents
-    @cooling_agents.setter
-    def cooling_agents(self, cooling_agents):
-        bst.HeatUtility.cooling_agents = cooling_agents
-        
-    @property
-    def heating_agents(self) -> list[bst.UtilityAgent]:
-        """All heating utilities available."""
-        return bst.HeatUtility.heating_agents
-    @heating_agents.setter
-    def heating_agents(self, heating_agents):
-        bst.HeatUtility.heating_agents = heating_agents
-        
-    @property
-    def stream_utility_prices(self) -> dict[str, float]:
-        """Price of stream utilities [USD/kg] which are defined as 
-        inlets and outlets to unit operations."""
-        return bst.stream_utility_prices
-    @stream_utility_prices.setter
-    def stream_utility_prices(self, stream_utility_prices):
-        bst.stream_utility_prices = stream_utility_prices
-    
-    @property
-    def impact_indicators(self) -> dict[str, str]:
-        """User-defined impact indicators and their units of measure."""
-        return bst.impact_indicators
-    @impact_indicators.setter
-    def impact_indicators(self, impact_indicators):
-        bst.impact_indicators = impact_indicators
-    
-    @property
-    def electricity_price(self) -> float:
-        """Electricity price [USD/kWhr]"""
-        return bst.PowerUtility.price
-    @electricity_price.setter
-    def electricity_price(self, electricity_price):
-        """Electricity price [USD/kWhr]"""
-        bst.PowerUtility.price = electricity_price
-    
     def set_thermo(self, thermo: tmo.Thermo|Iterable[str|tmo.Chemical], 
-                   cache: Optional[bool]=None,
-                   skip_checks: Optional[bool]=False, 
-                   ideal: Optional[bool]=False):
+            mixture: Optional[Mixture]=None,
+            Gamma: Optional[type[ActivityCoefficients]]=None,
+            Phi: Optional[type[FugacityCoefficients]]=None,
+            PCF: Optional[type[PoyintingCorrectionFactors]]=None,
+            cache: Optional[bool]=None,
+            skip_checks: Optional[bool]=False, 
+            ideal: Optional[bool]=False,
+        ):
         """
         Set the default :class:`~thermosteam.Thermo` object. If `thermo` is 
         not a :class:`~thermosteam.Thermo` object, an attempt is made to 
@@ -167,17 +123,24 @@ class ProcessSettings:
         ----------
         thermo : 
             Thermodynamic property package.
+        Gamma :
+            Class for computing activity coefficients.
+        Phi : 
+            Class for computing fugacity coefficients.
+        PCF : 
+            Class for computing poynting correction factors.
         cache : 
-            Wether or not to use cached chemicals.
+            Whether or not to use cached chemicals.
         skip_checks : 
             Whether to skip checks for missing or invalid properties.
         ideal :
             Whether to use ideal phase equilibrium and mixture property 
             algorithms.
-            
+        
         """
         if not isinstance(thermo, (tmo.Thermo, tmo.IdealThermo)):
-            thermo = tmo.Thermo(thermo, cache=cache, skip_checks=skip_checks)
+            thermo = tmo.Thermo(thermo, mixture=mixture, cache=cache, skip_checks=skip_checks,
+                                Gamma=Gamma, Phi=Phi, PCF=PCF)
         if ideal: thermo = thermo.ideal()
         self._thermo = thermo
     
