@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # BioSTEAM: The Biorefinery Simulation and Techno-Economic Analysis Modules
-# Copyright (C) 2020-2021, Yoel Cortes-Pena <yoelcortes@gmail.com>
+# Copyright (C) 2020-2023, Yoel Cortes-Pena <yoelcortes@gmail.com>
 # 
 # This module is under the UIUC open-source license. See 
 # github.com/BioSTEAMDevelopmentGroup/biosteam/blob/master/LICENSE.txt
@@ -8,6 +8,7 @@
 """
 """
 from __future__ import annotations
+import pandas as pd
 import numpy as np
 import thermosteam as tmo
 import flexsolve as flx
@@ -18,7 +19,9 @@ from . import units_of_measure as thermo_units
 from .exceptions import DimensionError, InfeasibleRegion
 from chemicals.elements import array_to_atoms, symbol_to_index
 from . import utils
+from .indexer import nonzeros
 from typing import TYPE_CHECKING
+from ._phase import valid_phases
 if TYPE_CHECKING:
     from numpy.typing import NDArray
     from free_properties import property_array
@@ -109,9 +112,9 @@ class Stream:
     >>> s1.show(composition=True, flow='kg/hr') # Its also possible to show by composition
     Stream: s1
      phase: 'l', T: 298.15 K, P: 101325 Pa
-     composition: Water    0.667
-                  Ethanol  0.333
-                  -------  30 kg/hr
+     composition (%): Water    66.7
+                      Ethanol  33.3
+                      -------  30 kg/hr
     
     All flow rates are stored as an array in the `mol` attribute:
     
@@ -340,21 +343,22 @@ class Stream:
     def __reduce__(self):
         return self.from_data, (self.get_data(), self._ID, self._price, self.characterization_factors, self._thermo)
 
-    def rescale(self, ratio):
+    def scale(self, scale):
         """
-        Rescale flow rate by given ratio.
+        Multiply flow rate by given scale.
         
         Examples
         --------
         >>> import thermosteam as tmo
         >>> tmo.settings.set_thermo(['Water', 'Ethanol'], cache=True)
         >>> s1 = tmo.Stream('s1', Water=1)
-        >>> s1.rescale(100)
+        >>> s1.scale(100)
         >>> s1.F_mol
         100.0
         
         """
-        self._imol._data[:] *= ratio
+        self._imol._data[:] *= scale
+    rescale = scale
 
     def reset_flow(self, phase=None, units=None, total_flow=None, **chemical_flows):
         """
@@ -369,8 +373,8 @@ class Stream:
         >>> s1.show('cwt')
         Stream: s1
          phase: 'g', T: 298.15 K, P: 101325 Pa
-         composition: Ethanol  1
-                      -------  2 kg/hr
+         composition (%): Ethanol  100
+                          -------  2 kg/hr
         
         """
         imol = self._imol
@@ -900,6 +904,20 @@ class Stream:
         """Inlet location."""
         return self._sink
 
+    def get_downstream_units(self, ends=None, facilities=True):
+        """Return a set of all units downstream."""
+        sink = self._sink
+        units = sink.get_downstream_units(ends, facilities)
+        units.add(sink)
+        return units
+    
+    def get_upstream_units(self, ends=None, facilities=True):
+        """Return a set of all units upstream."""
+        source = self._source
+        units = source.get_upstream_units(ends, facilities)
+        units.add(source)
+        return units
+
     @property
     def thermal_condition(self) -> tmo.ThermalCondition:
         """
@@ -1088,6 +1106,9 @@ class Stream:
     def Hnet(self) -> float:
         """Total enthalpy flow rate (including heats of formation) [kJ/hr]."""
         return self.H + self.Hf
+    @Hnet.setter
+    def Hnet(self, Hnet):
+        self.H = Hnet - self.Hf
     
     @property
     def Hf(self) -> float:
@@ -1132,11 +1153,11 @@ class Stream:
             calculate = getattr(self.mixture, name)
             if nophase:
                 property_cache[name] = value = calculate(
-                    composition, *self.thermal_condition
+                    composition, *self._thermal_condition
                 )
             else:
                 property_cache[name] = value = calculate(
-                    phase, composition, *self.thermal_condition
+                    phase, composition, *self._thermal_condition
                 )
             return value * total if flow else value
     
@@ -1336,10 +1357,18 @@ class Stream:
             self._imol.separate_out(other._imol)
             if energy_balance: self.H = H_new
     
-    def mix_from(self, others, energy_balance=True, vle=False):
+    def mix_from(self, others, energy_balance=True, vle=False, Q=0.):
         """
         Mix all other streams into this one, ignoring its initial contents.
         
+        Notes
+        -----
+        When streams at different pressures are mixed, BioSTEAM assumes valves 
+        reduce the pressure of the streams being mixed to prevent backflow 
+        (pressure needs to decrease in the direction of flow according to 
+        Bernoulli's principle). The outlet pressure will be the minimum pressure
+        of all streams being mixed.
+        
         Examples
         --------
         Mix two streams with the same thermodynamic property package:
@@ -1381,33 +1410,40 @@ class Stream:
          flow: 0
         
         """
-        others = [i for i in others if i]
-        N_others = len(others)
-        if N_others == 0:
+        streams = []
+        isa = isinstance
+        for i in others:
+            if isa(i, Stream):
+                if not i.isempty(): streams.append(i)
+            else: 
+                Q += i.heat # Must be a heat or power object, assume power turns to heat
+        N_streams = len(streams)
+        if N_streams == 0:
             self.empty()
-        elif N_others == 1:
-            self.copy_like(others[0])
-        elif vle:
-            phases = ''.join([i.phase for i in others])
-            self.phases = tuple(set(phases))
-            self._imol.mix_from([i._imol for i in others])
-            if energy_balance: 
-                H = sum([i.H for i in others])
-                self.vle(H=H, P=self.P)
-            else:
-                self.vle(T=self.T, P=self.P)
+        elif N_streams == 1:
+            self.copy_like(streams[0])
         else:
-            self.P = min([i.P for i in others])
-            if energy_balance: H = sum([i.H for i in others])
-            self._imol.mix_from([i._imol for i in others])
-            if energy_balance and not self.isempty():
-                try:
-                    self.H = H
-                except:
-                    phases = ''.join([i.phase for i in others])
-                    self.phases = tuple(set(phases))
-                    self._imol.mix_from([i._imol for i in others])
-                    self.H = H
+            P = min([i.P for i in streams])
+            if vle:
+                phases = ''.join([i.phase for i in streams])
+                self.phases = tuple(set(phases))
+                self._imol.mix_from([i._imol for i in streams])
+                if energy_balance: 
+                    H = sum([i.H for i in streams], Q)
+                    self.vle(H=H, P=P)
+                else:
+                    self.vle(T=self.T, P=P)
+            else:
+                if energy_balance: H = sum([i.H for i in streams], Q)
+                self._imol.mix_from([i._imol for i in streams])
+                if energy_balance and not self.isempty():
+                    try:
+                        self.H = H
+                    except:
+                        phases = ''.join([i.phase for i in streams])
+                        self.phases = tuple(set(phases))
+                        self._imol.mix_from([i._imol for i in streams])
+                        self.H = H
                 
     def split_to(self, s1, s2, split, energy_balance=True):
         """
@@ -1582,7 +1618,7 @@ class Stream:
 
         """
         if isinstance(other, tmo.MultiStream):
-            phase = other.phase
+            phase = other.phases[0]
             if len(phase) == 1:
                 imol = other._imol.to_chemical_indexer(phase)
             else:
@@ -1608,9 +1644,19 @@ class Stream:
         Stream: s1
          phase: 'l', T: 300 K, P: 101325 Pa
          flow (kg/hr): Water  2
+        
         """
         self._thermal_condition.copy_like(other._thermal_condition)
     
+    def copy_phase(self, other):
+        """Copy phase from another stream."""
+        try:
+            self._imol._phase._phase = other._imol._phase._phase
+        except AttributeError as e:
+            if isinstance(other, tmo.MultiStream): 
+                raise ValueError('cannot copy phase from stream with multiple phases')
+            raise e from None
+    
     def copy_flow(self, 
                   other: Stream, 
                   IDs: Optional[Sequence[str]|str]=..., *,
@@ -1878,18 +1924,21 @@ class Stream:
     @property
     def vle(self) -> eq.VLE:
         """An object that can perform vapor-liquid equilibrium on the stream."""
+        if self.phase == 's': self.phase = 'l'
         self.phases = ('g', 'l')
         return self.vle
 
     @property
     def lle(self) -> eq.LLE:
         """An object that can perform liquid-liquid equilibrium on the stream."""
+        if self.phase not in ('l', 'L'): self.phase = 'l'
         self.phases = ('L', 'l')
         return self.lle
     
     @property
     def sle(self) -> eq.SLE:
         """An object that can perform solid-liquid equilibrium on the stream."""
+        if self.phase not in ('l', 's'): self.phase = 'l'
         self.phases = ('s', 'l')
         return self.sle
 
@@ -1932,10 +1981,13 @@ class Stream:
                 return data
             else: # Perform VLE on each liquid phase
                 vle(T=T, P=P)
+                no_vapor = (data[1] == 0).all()
                 data[2], data[0] = data[0].copy(), data[2].copy()
                 vle(T=T, P=P)
+                if no_vapor and (data[1] == 0).all(): 
+                    done[0] = True # No VLE
             return data.copy()
-        data[:] = flx.fixed_point(f, data / total_flow, xtol=1e-3, checkiter=False) * total_flow
+        data[:] = flx.fixed_point(f, data / total_flow, xtol=1e-3, checkiter=False, checkconvergence=False, convergenceiter=10) * total_flow
 
     @property
     def vle_chemicals(self) -> list[tmo.Chemical]:
@@ -2337,7 +2389,7 @@ class Stream:
                 liquid.mol[index] = 0
                 ms.H = H 
                 return ms.T
-            flx.wegstein(equilibrium_approximation, T)
+            flx.wegstein(equilibrium_approximation, T, xtol=1e-4, maxiter=100)
         else:
             f_l = F_l(x, T)
             y = f_l / P
@@ -2421,15 +2473,96 @@ class Stream:
             if layout.isdigit():
                 N = int(layout)
         return flow, composition, N
+
+    def _info_str(self, T_units, P_units, flow_units, composition, N_max, all_IDs, indexer, factor):
+        basic_info = self._basic_info()
+        basic_info += self._info_phaseTP(self.phase, T_units, P_units)
+        if N_max == 0:
+            return basic_info[:-1]
+        N_IDs = len(all_IDs)
+        if N_IDs == 0:
+            return basic_info + ' flow: 0' 
+            
+        # Remaining lines (all flow rates)
+        flow_array = factor * indexer[all_IDs]
+        if composition:
+            total_flow = flow_array.sum()
+            beginning = " composition (%): "
+            new_line = '\n' + len(beginning) * ' '
+            flow_array = 100 * flow_array/total_flow
+        else:
+            beginning = f' flow ({flow_units}): '
+            new_line = '\n' + len(beginning) * ' '
+        flow_rates = ''
+        too_many_chemicals = N_IDs > N_max
+        if not too_many_chemicals: N_max = N_IDs
+        lengths = [len(i) for i in all_IDs[:N_max]]
+        maxlen = max(lengths) + 2
+        for i in range(N_max):
+            spaces = ' ' * (maxlen - lengths[i])
+            if i: flow_rates += new_line
+            flow_rates += all_IDs[i] + spaces + f'{flow_array[i]:.3g}'
+        if too_many_chemicals: 
+            spaces = ' ' * (maxlen - 3)
+            flow_rates += new_line + '...' + spaces + f'{flow_array[N_max:].sum():.3g}'
+        if composition:
+            dashes = '-' * (maxlen - 2)
+            flow_rates += f"{new_line}{dashes}  {total_flow:.3g} {flow_units}"
+        return (basic_info 
+              + beginning
+              + flow_rates)
     
-    def _info(self, layout, T, P, flow, composition, N, IDs):
+    def _info_df(self, T_units, P_units, flow_units, composition, N_max, all_IDs, indexer, factor):
+        if not all_IDs:
+            return pd.DataFrame([0], columns=[self.ID.replace('_', ' ')], index=['Flow'])
+        T = thermo_units.convert(self.T, 'K', T_units)
+        P = thermo_units.convert(self.P, 'Pa', P_units)
+        data = []
+        index = []
+        index.append((f"Temperature [{T_units}]", ''))
+        data.append(f"{T:.3g}")
+        index.append((f"Pressure [{P_units}]", ''))
+        data.append(f"{P:.6g}")
+        for phase in self.phases:
+            if indexer.data.ndim == 2:
+                flow_array = factor * indexer[phase, all_IDs]
+            else:
+                flow_array = factor * indexer[all_IDs]
+            phase = valid_phases[phase]
+            if phase.islower(): phase = phase.capitalize()
+            if composition:
+                total_flow = flow_array.sum()
+                index.append((f"{phase} [{flow_units}]", ''))
+                data.append(f"{total_flow:.3g}")
+                comp_array = 100 * flow_array / total_flow
+                for i, (ID, comp) in enumerate(zip(all_IDs, comp_array)):
+                    if not comp: continue
+                    if i >= N_max:
+                        index.append(("Composition [%]", '(remainder)'))
+                        data.append(f"{comp_array[N_max:].sum():.3g}")
+                        break
+                    else:
+                        index.append(("Composition [%]", ID))
+                        data.append(f"{comp:.3g}")
+            else:   
+                for i, (ID, flow) in enumerate(zip(all_IDs, flow_array)):
+                    if not flow: continue
+                    if i >= N_max:
+                        index.append((f"{phase} [{flow_units}]", '(remainder)'))
+                        data.append(f"{flow_array[N_max:].sum():.3g}")
+                        break
+                    else:
+                        index.append((f"{phase} [{flow_units}]", ID))
+                        data.append(f"{flow:.3g}")
+        return pd.DataFrame(data, columns=[self.ID.replace('_', ' ')], 
+                            index=pd.MultiIndex.from_tuples(index))
+        
+    def _info(self, layout, T, P, flow, composition, N, IDs, df=False):
         """Return string with all specifications."""
         flow, composition, N = self._translate_layout(layout, flow, composition, N)
-        from .indexer import nonzeros
-        basic_info = self._basic_info()
         if not IDs:
             IDs = self.chemicals.IDs
-            data = self.imol.data
+            data = self.mol
         else:
             data = self.imol[IDs]
         IDs, data = nonzeros(IDs, data)
@@ -2439,44 +2572,43 @@ class Stream:
         P_units = P or display_units.P
         flow_units = flow or display_units.flow
         N_max = display_units.N if N is None else N
-        basic_info += self._info_phaseTP(self.phase, T_units, P_units)
-        if N_max == 0:
-            return basic_info[:-1]
-        composition = display_units.composition if composition is None else composition 
-        N_IDs = len(IDs)
-        if N_IDs == 0:
-            return basic_info + ' flow: 0' 
-        
-        # Start of third line (flow rates)
+        composition = display_units.composition if composition is None else composition
         name, factor = self._get_flow_name_and_factor(flow_units)
         indexer = getattr(self, 'i' + name)
-            
-        # Remaining lines (all flow rates)
-        flow_array = factor * indexer[IDs]
-        if composition:
-            total_flow = flow_array.sum()
-            beginning = " composition: "
-            new_line = '\n' + 14 * ' '
-            flow_array = flow_array/total_flow
+        return (self._info_df if df else self._info_str)(T_units, P_units, flow_units, composition, N_max, IDs, indexer, factor)
+    
+    def _get_tooltip_string(self, format, full):
+        if format not in ('html', 'svg'): return ''
+        if self.isempty():
+            tooltip = '(empty)'
+        elif format == 'html' and full:
+            df = self._info(None, None, None, None, None, None, None, df=True)
+            tooltip = (
+                " " + # makes sure graphviz does not try to parse the string as HTML
+                df.to_html(justify='unset'). # unset makes sure that table header style can be overwritten in CSS
+                replace("\n", "").replace("  ", "") # makes sure tippy.js does not add any whitespaces
+            )
         else:
-            beginning = f' flow ({flow_units}): '
-            new_line = '\n' + len(beginning) * ' '
-        flow_rates = ''
-        lengths = [len(i) for i in IDs]
-        maxlen = max(lengths) + 2
-        too_many_chemicals = N_IDs > N_max
-        N = N_max if too_many_chemicals else N_IDs
-        for i in range(N):
-            spaces = ' ' * (maxlen - lengths[i])
-            if i: flow_rates += new_line
-            flow_rates += IDs[i] + spaces + f'{flow_array[i]:.3g}'
-        if too_many_chemicals: flow_rates += new_line + '...'
-        if composition:
-            dashes = '-' * (maxlen - 2)
-            flow_rates += f"{new_line}{dashes}  {total_flow:.3g} {flow_units}"
-        return (basic_info 
-              + beginning
-              + flow_rates)
+            newline = '<br>' if format == 'html' else '\n'
+            display_units = self.display_units
+            T_units = display_units.T
+            P_units = display_units.P
+            flow_units = display_units.flow
+            T = thermo_units.convert(self.T, 'K', T_units)
+            P = thermo_units.convert(self.P, 'Pa', P_units)
+            tooltip = (
+                f"Temperature: {T:.3g} {T_units}{newline}"
+                f"Pressure: {P:.6g} {P_units}"
+            )
+            for phase in self.phases:
+                stream = self[phase] if self.imol.data.ndim == 2 else self
+                flow = stream.get_total_flow(flow_units)
+                phase = valid_phases[phase]
+                if phase.islower(): phase = phase.capitalize()
+                tooltip += f"{newline}{phase} flow: {flow:.3g} {flow_units}"
+            if format == 'html':
+                tooltip = " " + tooltip
+        return tooltip
 
     def show(self, 
              layout: Optional[str]=None,
@@ -2485,7 +2617,8 @@ class Stream:
              flow: Optional[str]=None, 
              composition: Optional[bool]=None, 
              N: Optional[int]=None, 
-             IDs: Optional[Sequence[str]]=None):
+             IDs: Optional[Sequence[str]]=None,
+             df: Optional[bool]=None):
         """
         Print all specifications.
         
@@ -2508,9 +2641,11 @@ class Stream:
             Number of compounds to display.
         IDs : 
             IDs of compounds to display. Defaults to all chemicals.
+        df :
+            Whether to return a pandas DataFrame.
         
         """
-        print(self._info(layout, T, P, flow, composition, N, IDs))
+        print(self._info(layout, T, P, flow, composition, N, IDs, df))
     _ipython_display_ = show
     
     def print(self, units: Optional[str]=None):
