@@ -1,12 +1,13 @@
 # -*- coding: utf-8 -*-
 # BioSTEAM: The Biorefinery Simulation and Techno-Economic Analysis Modules
-# Copyright (C) 2020-2021, Yoel Cortes-Pena <yoelcortes@gmail.com>
+# Copyright (C) 2020-2023, Yoel Cortes-Pena <yoelcortes@gmail.com>
 # 
 # This module is under the UIUC open-source license. See 
 # github.com/BioSTEAMDevelopmentGroup/biosteam/blob/master/LICENSE.txt
 # for license details.
 """
 """
+import numpy as np
 import thermosteam as tmo
 from . import equilibrium as eq
 from ._chemical import Chemical
@@ -16,7 +17,6 @@ from .utils import read_only, cucumber
 
 __all__ = ('Thermo', 'IdealThermo')
 
-
 @cucumber # Just means you can pickle it
 @read_only
 class Thermo:
@@ -55,7 +55,7 @@ class Thermo:
         ),
         Gamma=DortmundActivityCoefficients,
         Phi=IdealFugacityCoefficients,
-        PCF=IdealPoyintingCorrectionFactors
+        PCF=MockPoyintingCorrectionFactors
     )
     
     Note that the Dortmund-UNIFAC is the default activity coefficient model. 
@@ -108,7 +108,7 @@ class Thermo:
         ),
         Gamma=DortmundActivityCoefficients,
         Phi=IdealFugacityCoefficients,
-        PCF=IdealPoyintingCorrectionFactors
+        PCF=MockPoyintingCorrectionFactors
     )
     
     Attributes
@@ -128,11 +128,14 @@ class Thermo:
     __slots__ = ('chemicals', 'mixture', 'Gamma', 'Phi', 'PCF', '_ideal', '_original_thermo') 
     
     def __init__(self, chemicals, mixture=None,
-                 Gamma=eq.DortmundActivityCoefficients,
-                 Phi=eq.IdealFugacityCoefficients,
-                 PCF=eq.IdealPoyintingCorrectionFactors,
+                 Gamma=None,
+                 Phi=None,
+                 PCF=None,
                  cache=None,
                  skip_checks=False):
+        if Gamma is None: Gamma = eq.DortmundActivityCoefficients
+        if Phi is None: Phi = eq.IdealFugacityCoefficients
+        if PCF is None: PCF = eq.MockPoyintingCorrectionFactors
         if not isinstance(chemicals, Chemicals): chemicals = Chemicals(chemicals, cache)
         if not mixture:
             mixture = Mixture.from_chemicals(chemicals)
@@ -167,8 +170,9 @@ class Thermo:
     def subset(self, chemicals):
         if chemicals is self.chemicals: return self
         if not isinstance(chemicals, Chemicals): chemicals = Chemicals(chemicals)
+        ndarray = np.ndarray
         groups = [(name, index) for name, index in self.chemicals._index.items() 
-                  if isinstance(index, list)]
+                  if isinstance(index, ndarray)]
         chemicals.compile(skip_checks=True)
         CASs = self.chemicals.CASs
         for name, index in groups:
@@ -274,7 +278,7 @@ class IdealThermo:
     
     Gamma = eq.IdealActivityCoefficients
     Phi = eq.IdealFugacityCoefficients
-    PCF = eq.IdealPoyintingCorrectionFactors
+    PCF = eq.MockPoyintingCorrectionFactors
     as_chemical = Thermo.as_chemical
     __enter__ = Thermo.__enter__
     __exit__ = Thermo.__exit__
@@ -296,8 +300,9 @@ class IdealThermo:
     def subset(self, chemicals):
         if chemicals is self.chemicals: return self
         if not isinstance(chemicals, Chemicals): chemicals = Chemicals(chemicals)
+        ndarray = np.ndarray
         groups = [(name, index) for name, index in self.chemicals._index.items() 
-                  if isinstance(index, list)]
+                  if isinstance(index, ndarray)]
         chemicals.compile(skip_checks=True)
         CASs = self.chemicals.CASs
         for name, index in groups:
