@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # BioSTEAM: The Biorefinery Simulation and Techno-Economic Analysis Modules
-# Copyright (C) 2020-2021, Yoel Cortes-Pena <yoelcortes@gmail.com>
+# Copyright (C) 2020-2023, Yoel Cortes-Pena <yoelcortes@gmail.com>
 # 
 # This module is under the UIUC open-source license. See 
 # github.com/BioSTEAMDevelopmentGroup/biosteam/blob/master/LICENSE.txt
@@ -17,7 +17,7 @@ from .equilibrium import Equilibrium
 from .dew_point import DewPointCache
 from .bubble_point import BubblePointCache
 from .fugacity_coefficients import IdealFugacityCoefficients
-from .poyinting_correction_factors import IdealPoyintingCorrectionFactors
+from .poyinting_correction_factors import MockPoyintingCorrectionFactors
 from . import activity_coefficients as ac
 from .. import functional as fn
 from ..utils import Cache
@@ -26,16 +26,14 @@ import numpy as np
 __all__ = ('VLE', 'VLECache')
 
 # @njit(cache=True)
-def xV_iter(xV, Psat_over_P_phi, T, z, z_light, z_heavy, f_gamma, gamma_args, f_pcf, pcf_args):
+def xV_iter(xV, pcf_Psat_over_P_phi, T, P, z, z_light, z_heavy, f_gamma, gamma_args):
     xV = xV.copy()
     x = xV[:-1]
     V = xV[-1]
     x[x < 0.] = 0.
     x = fn.normalize(x)
-    Ks = Psat_over_P_phi * f_gamma(x, T, *gamma_args) * f_pcf(x, T, *pcf_args)
-    V = binary.solve_phase_fraction_Rashford_Rice(z, Ks, V,
-                                       z_light,
-                                       z_heavy)
+    Ks = pcf_Psat_over_P_phi * f_gamma(x, T, *gamma_args)
+    V = binary.solve_phase_fraction_Rashford_Rice(z, Ks, V, z_light, z_heavy)
     if V < 0.: V = 0.
     elif V > 1.: V = 1.
     xV[-1] = V
@@ -43,13 +41,13 @@ def xV_iter(xV, Psat_over_P_phi, T, z, z_light, z_heavy, f_gamma, gamma_args, f_
     return xV
 
 @njit(cache=True)    
-def xV_iter_2n(xV, Psat_over_P_phi, T, z, f_gamma, gamma_args, f_pcf, pcf_args):
+def xV_iter_2n(xV, pcf_Psat_over_P_phi, T, P, z, f_gamma, gamma_args):
     xV = xV.copy()
     x = xV[:-1]
     V = xV[-1]
     x[x < 0.] = 0.
     x = fn.normalize(x)
-    Ks = Psat_over_P_phi * f_gamma(x, T, *gamma_args) * f_pcf(x, T, *pcf_args)
+    Ks = pcf_Psat_over_P_phi * f_gamma(x, T, *gamma_args)
     V = binary.compute_phase_fraction_2N(z, Ks)
     if V < 0.: V = 0.
     elif V > 1.: V = 1.
@@ -58,13 +56,13 @@ def xV_iter_2n(xV, Psat_over_P_phi, T, z, f_gamma, gamma_args, f_pcf, pcf_args):
     return xV
 
 @njit(cache=True)
-def xV_iter_3n(xV, Psat_over_P_phi, T, z, f_gamma, gamma_args, f_pcf, pcf_args):
+def xV_iter_3n(xV, pcf_Psat_over_P_phi, T, P, z, f_gamma, gamma_args):
     xV = xV.copy()
     x = xV[:-1]
     V = xV[-1]
     x[x < 0.] = 0.
     x = fn.normalize(x)
-    Ks = Psat_over_P_phi * f_gamma(x, T, *gamma_args) * f_pcf(x, T, *pcf_args)
+    Ks = pcf_Psat_over_P_phi * f_gamma(x, T, *gamma_args)
     V = binary.compute_phase_fraction_3N(z, Ks)
     if V < 0.: V = 0.
     elif V > 1.: V = 1.
@@ -72,6 +70,11 @@ def xV_iter_3n(xV, Psat_over_P_phi, T, z, f_gamma, gamma_args, f_pcf, pcf_args):
     xV[:-1] = z/(1. + V * (Ks - 1.))
     return xV
 
+@njit(cache=True)
+def set_flows(vapor_mol, liquid_mol, index, vapor_data, total_data):
+    vapor_mol[index] = vapor_data
+    liquid_mol[index] = total_data - vapor_data
+
 class VLE(Equilibrium, phases='lg'):
     """
     Create a VLE object that performs vapor-liquid equilibrium when called.
@@ -220,9 +223,9 @@ class VLE(Equilibrium, phases='lg'):
     >>> vle(S=S, T=363.88)
     >>> vle 
     VLE(imol=MolarFlowIndexer(
-            g=[('Water', 128.1), ('Ethanol', 26.46), ('Methanol', 33.59), ('Propanol', 0.8979), ('O2', 10)],
-            l=[('Water', 175.9), ('Ethanol', 3.536), ('Methanol', 6.406), ('Propanol', 0.1021), ('Glucose', 5)]),
-        thermal_condition=ThermalCondition(T=363.88, P=106576))
+            g=[('Water', 128.1), ('Ethanol', 26.46), ('Methanol', 33.59), ('Propanol', 0.8978), ('O2', 10)],
+            l=[('Water', 175.9), ('Ethanol', 3.538), ('Methanol', 6.409), ('Propanol', 0.1022), ('Glucose', 5)]),
+        thermal_condition=ThermalCondition(T=363.88, P=106583))
 
     The presence of a non-partitioning gaseous chemical will result in some
     evaporation, even if the tempeture is below the saturated bubble point:
@@ -486,15 +489,20 @@ class VLE(Equilibrium, phases='lg'):
     ### Single component equilibrium case ###
         
     def _set_thermal_condition_chemical(self, T, P):
-        # Either liquid or gas
-        Psat = self._chemical.Psat(T)
-        tol = 1e-3
-        if P < Psat - tol:
+        chemical = self._chemical
+        if T >= chemical.Tc: 
             self._liquid_mol[self._index] = 0
             self._vapor_mol[self._index] = self._mol_vle
-        elif P > Psat + tol:
-            self._liquid_mol[self._index] = self._mol_vle
-            self._vapor_mol[self._index] = 0
+        else:
+            # Either liquid or gas
+            Psat = chemical.Psat(T)
+            tol = 1e-3
+            if P < Psat - tol:
+                self._liquid_mol[self._index] = 0
+                self._vapor_mol[self._index] = self._mol_vle
+            elif P > Psat + tol:
+                self._liquid_mol[self._index] = self._mol_vle
+                self._vapor_mol[self._index] = 0
     
     def _set_TV_chemical(self, T, V):
         # Set vapor fraction
@@ -516,9 +524,10 @@ class VLE(Equilibrium, phases='lg'):
         thermo = self._thermo
         phase_data = self._phase_data
         mixture = thermo.mixture
+        chemical = self._chemical
         
         # Set temperature in equilibrium
-        self._T = self._thermal_condition.T = T = self._chemical.Tsat(P, check_validity=False)
+        self._T = self._thermal_condition.T = T = chemical.Tsat(P, check_validity=False) 
         
         # Check if super heated vapor
         vapor_mol[index] = mol
@@ -557,7 +566,8 @@ class VLE(Equilibrium, phases='lg'):
         liquid_mol[index] = 0
         H_dew = mixture.xH(phase_data, T, P)
         if H >= H_dew:
-            self._thermal_condition.T = mixture.xsolve_T_at_HP(phase_data, H, T, P)
+            raise NotImplementedError('cannot solve for pressure yet')
+            self._thermal_condition.P = mixture.xsolve_P_at_HT(phase_data, H, T, P)
             return
 
         # Check if subcooled liquid
@@ -565,7 +575,8 @@ class VLE(Equilibrium, phases='lg'):
         liquid_mol[index] = mol
         H_bubble = mixture.xH(phase_data, T, P)
         if H <= H_bubble:
-            self._thermal_condition.T = mixture.xsolve_T_at_HP(phase_data, H, T, P)
+            raise NotImplementedError('cannot solve for pressure yet')
+            self._thermal_condition.P = mixture.xsolve_P_at_HT(phase_data, H, T, P)
             return
         
         # Adjust vapor fraction accordingly
@@ -581,9 +592,10 @@ class VLE(Equilibrium, phases='lg'):
         thermo = self._thermo
         phase_data = self._phase_data
         mixture = thermo.mixture
+        chemical = self._chemical
         
         # Set temperature in equilibrium
-        self._T = self._thermal_condition.T = T = self._chemical.Tsat(P, check_validity=False)
+        self._T = self._thermal_condition.T = T = chemical.Tsat(P, check_validity=False)
         
         # Check if super heated vapor
         vapor_mol[index] = mol
@@ -622,7 +634,8 @@ class VLE(Equilibrium, phases='lg'):
         liquid_mol[index] = 0
         S_dew = mixture.xS(phase_data, T, P)
         if S >= S_dew:
-            self._thermal_condition.T = mixture.xsolve_T_at_SP(phase_data, S, T, P)
+            raise NotImplementedError('cannot solve for pressure yet')
+            self._thermal_condition.P = mixture.xsolve_P_at_ST(phase_data, S, T, P)
             return
 
         # Check if subcooled liquid
@@ -630,7 +643,8 @@ class VLE(Equilibrium, phases='lg'):
         liquid_mol[index] = mol
         S_bubble = mixture.xS(phase_data, T, P)
         if S <= S_bubble:
-            self._thermal_condition.T = mixture.xsolve_T_at_SP(phase_data, S, T, P)
+            raise NotImplementedError('cannot solve for pressure yet')
+            self._thermal_condition.P = mixture.xsolve_P_at_ST(phase_data, S, T, P)
             return
         
         # Adjust vapor fraction accordingly
@@ -696,9 +710,7 @@ class VLE(Equilibrium, phases='lg'):
         dP = (P_bubble - P_dew)
         V = (P - P_dew) / dP if dP > 1. else 0.5
         self._refresh_v(V, y_bubble)
-        v = self._solve_v(T, P)
-        self._vapor_mol[self._index] = v
-        self._liquid_mol[self._index] = self._mol_vle - v
+        set_flows(self._vapor_mol, self._liquid_mol, self._index, self._solve_v(T, P), self._mol_vle)
         
     def set_TV(self, T, V):
         self._setup()
@@ -752,8 +764,7 @@ class VLE(Equilibrium, phases='lg'):
                     v = self._v
             
             self._P = thermal_condition.P = P
-            self._vapor_mol[self._index] = v
-            self._liquid_mol[self._index] = mol - v
+            set_flows(self._vapor_mol, self._liquid_mol, self._index, v, mol)
             self._H_hat = self.mixture.xH(self._phase_data, T, P) / self._F_mass
 
     def set_TH(self, T, H):
@@ -774,7 +785,7 @@ class VLE(Equilibrium, phases='lg'):
         H_dew = self.mixture.xH(phase_data, T, P_dew)
         dH_dew = (H - H_dew)
         if dH_dew >= 0:
-            raise InfeasibleRegion(f'T={T:.3g} and H={H:.3g}')
+            raise NotImplementedError('cannot solve for pressure yet')
 
         # Check if subcooled liquid
         P_bubble, y_bubble = self._bubble_point.solve_Py(self._z, T)
@@ -783,7 +794,7 @@ class VLE(Equilibrium, phases='lg'):
         H_bubble = self.mixture.xH(phase_data, T, P_bubble)
         dH_bubble = (H - H_bubble)
         if dH_bubble <= 0:
-            raise InfeasibleRegion(f'T={T:.3g} and H={H:.3g}')
+            raise NotImplementedError('cannot solve for pressure yet')
 
         # Guess overall vapor fraction, and vapor flow rates
         V = dH_bubble/(H_dew - H_bubble)
@@ -821,7 +832,7 @@ class VLE(Equilibrium, phases='lg'):
         S_dew = self.mixture.xS(phase_data, T, P_dew)
         dS_dew = (S - S_dew)
         if dS_dew >= 0:
-            raise InfeasibleRegion(f'T={T:.3g} and S={S:.3g}')
+            raise NotImplementedError('cannot solve for pressure yet')
 
         # Check if subcooled liquid
         P_bubble, y_bubble = self._bubble_point.solve_Py(self._z, T)
@@ -830,7 +841,7 @@ class VLE(Equilibrium, phases='lg'):
         S_bubble = self.mixture.xS(phase_data, T, P_bubble)
         dS_bubble = (S - S_bubble)
         if dS_bubble <= 0:
-            raise InfeasibleRegion(f'T={T:.3g} and S={S:.3g}')
+            raise NotImplementedError('cannot solve for pressure yet')
 
         # Guess overall vapor fraction, and vapor flow rates
         V = dS_bubble/(S_dew - S_bubble)
@@ -878,8 +889,8 @@ class VLE(Equilibrium, phases='lg'):
             T_dew, x_dew = self._dew_point.solve_Tx(self._z, P)
             T_bubble, y_bubble = self._bubble_point.solve_Ty(self._z, P)
             self._refresh_v(V, y_bubble)
-            if self._F_mol_heavy: T_dew = self._dew_point.Tmax
-            if self._F_mol_light: T_bubble = self._bubble_point.Tmin
+            if self._F_mol_heavy: T_dew = 0.9 * T_dew + 0.1 * self._dew_point.Tmax
+            if self._F_mol_light: T_bubble = 0.9 * T_bubble + 0.1 * self._bubble_point.Tmin
             V_bubble = self._V_err_at_T(T_bubble, 0.)
             if V_bubble > V:
                 F_mol_vapor = self._F_mol * V
@@ -906,8 +917,7 @@ class VLE(Equilibrium, phases='lg'):
                 
                     v = self._v
             self._T = thermal_condition.T = T
-            vapor_mol[index] = v
-            liquid_mol[index] = mol - v
+            set_flows(vapor_mol, liquid_mol, index, v, mol)
             self._H_hat = self.mixture.xH(self._phase_data, T, P)/self._F_mass
     
     def set_PS(self, P, S, stacklevel=0):
@@ -929,7 +939,7 @@ class VLE(Equilibrium, phases='lg'):
         
         # Check if subcooled liquid
         T_bubble, y_bubble = self._bubble_point.solve_Ty(self._z, P)
-        if self._F_mol_light: T_bubble = self._bubble_point.Tmin
+        if self._F_mol_light: T_bubble = 0.9 * T_bubble + 0.1 * self._bubble_point.Tmin
         vapor_mol[index] = 0
         liquid_mol[index] = mol
         S_bubble = self.mixture.xS(self._phase_data, T_bubble, P)
@@ -940,7 +950,11 @@ class VLE(Equilibrium, phases='lg'):
         
         # Check if super heated vapor
         T_dew, x_dew = self._dew_point.solve_Tx(self._z, P)
-        if self._F_mol_heavy or T_dew < T_bubble: T_dew = self._dew_point.Tmax
+        if T_dew <= T_bubble: 
+            T_dew, T_bubble = T_bubble, T_dew
+            T_dew += 0.5
+            T_bubble -= 0.5
+        if self._F_mol_heavy: T_dew = 0.9 * T_dew + 0.1 * self._dew_point.Tmax
         vapor_mol[index] = mol
         liquid_mol[index] = 0
         S_dew = self.mixture.xS(self._phase_data, T_dew, P)
@@ -956,67 +970,68 @@ class VLE(Equilibrium, phases='lg'):
         F_mass = self._F_mass
         S_hat = S/F_mass
         S_hat_bubble = self._S_hat_err_at_T(T_bubble, 0.)
-        if S_hat_bubble > S_hat:
-            def f(V):
-                vapor_mol[index] = V * mol
-                liquid_mol[index] = (1 - V) * mol
-                return self.mixture.xS(self._phase_data, T_bubble, P)/self._F_mass - S_hat
-            self._T = thermal_condition.T = T_bubble
-            flx.IQ_interpolation(f,
-                0, 1., f(0), f(1.), V, self.V_tol, self.S_hat_tol, 
-                checkiter=False, checkbounds=False,
-                maxiter=self.maxiter,
-            )
-            return
-        S_hat_dew = self._S_hat_err_at_T(T_dew, 0.)
-        if S_hat_dew < S_hat:
-            def f(V):
-                vapor_mol[index] = V * mol
-                liquid_mol[index] = (1 - V) * mol
-                return self.mixture.xS(self._phase_data, T_dew, P)/self._F_mass - S_hat
-            self._T = thermal_condition.T = T_dew
-            flx.IQ_interpolation(f,
-                0, 1., f(0), f(1.), V, self.V_tol, self.S_hat_tol, 
-                checkiter=False, checkbounds=False,
-                maxiter=self.maxiter,
-            )
-            return
+        if (S_hat_bubble:=self._S_hat_err_at_T(T_bubble, 0.)) > S_hat:
+            T = T_bubble
+        elif (S_hat_dew:=self._S_hat_err_at_T(T_dew, 0.)) < S_hat:
+            T = T_dew
         else:
-            T = flx.IQ_interpolation(self._S_hat_err_at_T,
-                                     T_bubble, T_dew, 
-                                     S_hat_bubble - S_hat, S_hat_dew - S_hat,
-                                     self._T, self.T_tol, self.S_hat_tol,
-                                     (S_hat,), checkiter=False, checkbounds=False)
-            # Make sure S balance is correct
+            T = flx.IQ_interpolation(
+                self._S_hat_err_at_T, T_bubble, T_dew, 
+                S_hat_bubble - S_hat, S_hat_dew - S_hat,
+                self._T, self.T_tol, self.S_hat_tol,
+                (S_hat,), checkiter=False, checkbounds=False,
+                maxiter=self.maxiter
+            )
+        # Make sure energy balance is correct by vaporizing a fraction
+        # of the liquid or condensing a fraction of the vapor
+        self._T = thermal_condition.T = T
+        mol_liq = liquid_mol.copy()
+        mol_liq[:] = 0.
+        mol_liq[index] = liquid_mol[index]
+        mol_gas= vapor_mol.copy()
+        mol_gas[:] = 0.
+        mol_gas[index] = vapor_mol[index]
+        S_gas = self.mixture.S('g', mol_gas, T, P)
+        S_liq = self.mixture.S('l', mol_liq, T, P)
+        S_current = self.mixture.xS(self._phase_data, T, P)
+        if S_current > S: # Condense a fraction of the vapor
+            # Energy balance: S = f * S_condense + S_current
+            S_condense = self.mixture.S('l', mol_gas, T, P) - S_gas
             try:
-                self._T = thermal_condition.T = self.mixture.xsolve_T_at_SP(
-                    self._phase_data, S, T, P
-                )
-            except:
-                warn('VLE algorithm failed; resorting to fallback measures to ensure energy balance',
-                     category=RuntimeWarning, stacklevel=2+stacklevel)
-                def f(V):
-                    vapor_mol[index] = V * mol
-                    liquid_mol[index] = (1 - V) * mol
-                    return self.mixture.xS(self._phase_data, T, P)/self._F_mass - S_hat
-                self._T = thermal_condition.T = T
-                
-                y0 = f(0.)
-                if y0 > 0.:
-                    self._T = thermal_condition.T = self.mixture.xsolve_T_at_SP(
-                        self._phase_data, S, T, P
-                    )
+                f = (S - S_current) / S_condense
+            except: # Floating point errors
+                f = 0
+            else:
+                if f < 0.:
+                    f = 0.
+                elif f > 0.:
+                    if f > 1.: f = 1.
+                    condensed = f * mol_gas[index]
+                    liquid_mol[index] += condensed
+                    vapor_mol[index] -= condensed 
                 else:
-                    y1 = f(1.)
-                    if y1 < 0.:
-                        self._T = thermal_condition.T = self.mixture.xsolve_T_at_SP(
-                            self._phase_data, S, T, P
-                        )
-                    else:
-                        flx.IQ_interpolation(f,
-                            0., 1., y0, y1, self._V, self.V_tol, self.S_hat_tol,
-                            maxiter=self.maxiter, 
-                        )
+                    f = 0.
+        else: # Vaporize a fraction of the liquid
+            # Energy balance: S = f * S_vaporise + S_current
+            S_vaporise = self.mixture.S('g', mol_liq, T, P) - S_liq
+            try:
+                f = (S - S_current) / S_vaporise
+            except: # Floating point errors
+                f = 0
+            else:
+                if f < 0.:
+                    f = 0.
+                elif f > 0.:
+                    if f > 1.: f = 1.
+                    vaporised = f * mol_liq[index]
+                    vapor_mol[index] += vaporised
+                    liquid_mol[index] -= vaporised
+                else:
+                    f = 0.
+        if f == 0. or f == 1.:
+            self._T = thermal_condition.T = self.mixture.xsolve_T_at_SP(
+                self._phase_data, S, T, P
+            )
         self._S_hat = S_hat
     
     def set_PH(self, P, H, stacklevel=0):
@@ -1038,7 +1053,7 @@ class VLE(Equilibrium, phases='lg'):
         
         # Check if subcooled liquid
         T_bubble, y_bubble = self._bubble_point.solve_Ty(self._z, P)
-        if self._F_mol_light: T_bubble = self._bubble_point.Tmin
+        if self._F_mol_light: T_bubble = 0.9 * T_bubble + 0.1 * self._bubble_point.Tmin
         vapor_mol[index] = 0
         liquid_mol[index] = mol
         H_bubble = self.mixture.xH(self._phase_data, T_bubble, P)
@@ -1049,7 +1064,11 @@ class VLE(Equilibrium, phases='lg'):
         
         # Check if super heated vapor
         T_dew, x_dew = self._dew_point.solve_Tx(self._z, P)
-        if self._F_mol_heavy or T_dew < T_bubble: T_dew = self._dew_point.Tmax
+        if T_dew <= T_bubble: 
+            T_dew, T_bubble = T_bubble, T_dew
+            T_dew += 0.5
+            T_bubble -= 0.5
+        if self._F_mol_heavy: T_dew = 0.9 * T_dew + 0.1 * self._dew_point.Tmax
         vapor_mol[index] = mol
         liquid_mol[index] = 0
         H_dew = self.mixture.xH(self._phase_data, T_dew, P)
@@ -1064,68 +1083,70 @@ class VLE(Equilibrium, phases='lg'):
         
         F_mass = self._F_mass
         H_hat = H/F_mass
-        H_hat_bubble = self._H_hat_err_at_T(T_bubble, 0.)
-        if H_hat_bubble > H_hat:
-            def f(V):
-                vapor_mol[index] = V * mol
-                liquid_mol[index] = (1 - V) * mol
-                return self.mixture.xH(self._phase_data, T_bubble, P)/self._F_mass - H_hat
-            self._T = thermal_condition.T = T_bubble
-            flx.IQ_interpolation(f,
-                0, 1., f(0), f(1.), V, self.V_tol, self.H_hat_tol, 
-                checkiter=False, checkbounds=False,
-                maxiter=self.maxiter,
-            )
-            return
-        H_hat_dew = self._H_hat_err_at_T(T_dew, 0.)
-        if H_hat_dew < H_hat:
-            def f(V):
-                vapor_mol[index] = V * mol
-                liquid_mol[index] = (1 - V) * mol
-                return self.mixture.xH(self._phase_data, T_dew, P)/self._F_mass - H_hat
-            self._T = thermal_condition.T = T_dew
-            flx.IQ_interpolation(f,
-                0, 1., f(0), f(1.), V, self.V_tol, self.H_hat_tol, 
-                checkiter=False, checkbounds=False,
-                maxiter=self.maxiter,
-            )
-            return
+        
+        if (H_hat_bubble:=self._H_hat_err_at_T(T_bubble, 0.)) > H_hat:
+            T = T_bubble
+        elif (H_hat_dew:=self._H_hat_err_at_T(T_dew, 0.)) < H_hat:
+            T = T_dew
         else:
-            T = flx.IQ_interpolation(self._H_hat_err_at_T,
-                                     T_bubble, T_dew, 
-                                     H_hat_bubble - H_hat, H_hat_dew - H_hat,
-                                     self._T, self.T_tol, self.H_hat_tol,
-                                     (H_hat,), checkiter=False, checkbounds=False)
-            # Make sure H balance is correct
-            try:
-                self._T = thermal_condition.T = self.mixture.xsolve_T_at_HP(
-                    self._phase_data, H, T, P
-                )
-            except:
-                warn('VLE algorithm failed; resorting to fallback measures to ensure energy balance',
-                     category=RuntimeWarning, stacklevel=2+stacklevel)
-                def f(V):
-                    vapor_mol[index] = V * mol
-                    liquid_mol[index] = (1 - V) * mol
-                    return self.mixture.xH(self._phase_data, T, P)/self._F_mass - H_hat
-                self._T = thermal_condition.T = T
+            T = flx.IQ_interpolation(
+                self._H_hat_err_at_T, T_bubble, T_dew, 
+                H_hat_bubble - H_hat, H_hat_dew - H_hat,
+                self._T, self.T_tol, self.H_hat_tol,
+                (H_hat,), checkiter=False, checkbounds=False,
+                maxiter=self.maxiter
+            )
                 
-                y0 = f(0.)
-                if y0 > 0.:
-                    self._T = thermal_condition.T = self.mixture.xsolve_T_at_HP(
-                        self._phase_data, H, T, P
-                    )
+        # Make sure energy balance is correct by vaporizing a fraction
+        # of the liquid or condensing a fraction of the vapor
+        self._T = thermal_condition.T = T
+        mol_liq = liquid_mol.copy()
+        mol_liq[:] = 0.
+        mol_liq[index] = liquid_mol[index]
+        mol_gas= vapor_mol.copy()
+        mol_gas[:] = 0.
+        mol_gas[index] = vapor_mol[index]
+        H_gas = self.mixture.H('g', mol_gas, T, P)
+        H_liq = self.mixture.H('l', mol_liq, T, P)
+        H_current = self.mixture.xH(self._phase_data, T, P)
+        if H_current > H: # Condense a fraction of the vapor
+            # Energy balance: H = f * H_condense + H_current
+            H_condense = self.mixture.H('l', mol_gas, T, P) - H_gas
+            try:
+                f = (H - H_current) / H_condense
+            except: # Floating point errors
+                f = 0
+            else:
+                if f < 0.:
+                    f = 0.
+                elif f > 0.:
+                    if f > 1.: f = 1.
+                    condensed = f * mol_gas[index]
+                    liquid_mol[index] += condensed
+                    vapor_mol[index] -= condensed 
                 else:
-                    y1 = f(1.)
-                    if y1 < 0.:
-                        self._T = thermal_condition.T = self.mixture.xsolve_T_at_HP(
-                            self._phase_data, H, T, P
-                        )
-                    else:
-                        flx.IQ_interpolation(f,
-                            0., 1., y0, y1, self._V, self.V_tol, self.H_hat_tol,
-                            maxiter=self.maxiter, 
-                        )
+                    f = 0
+        else: # Vaporize a fraction of the liquid
+            # Energy balance: H = f * H_vaporise + H_current
+            H_vaporise = self.mixture.H('g', mol_liq, T, P) - H_liq
+            try:
+                f = (H - H_current) / H_vaporise
+            except: # Floating point errors
+                f = 0
+            else:
+                if f < 0.:
+                    f = 0.
+                elif f > 0.:
+                    if f > 1.: f = 1.
+                    vaporised = f * mol_liq[index]
+                    vapor_mol[index] += vaporised
+                    liquid_mol[index] -= vaporised
+                else:
+                    f = 0
+        if f == 0. or f == 1.:
+            self._T = thermal_condition.T = self.mixture.xsolve_T_at_HP(
+                self._phase_data, H, T, P
+            )
         self._H_hat = H_hat
     
     def _estimate_v(self, V, y_bubble):
@@ -1145,26 +1166,26 @@ class VLE(Equilibrium, phases='lg'):
             self._x = fn.normalize(l, l.sum() + self._F_mol_heavy)
     
     def _H_hat_err_at_T(self, T, H_hat):
-        self._vapor_mol[self._index] = self._solve_v(T, self._P)
-        self._liquid_mol[self._index] = self._mol_vle - self._v
+        set_flows(self._vapor_mol, self._liquid_mol, self._index, 
+                  self._solve_v(T, self._P), self._mol_vle)
         self._H_hat = self.mixture.xH(self._phase_data, T, self._P)/self._F_mass
         return self._H_hat - H_hat
     
     def _H_hat_err_at_P(self, P, H_hat):
-        self._vapor_mol[self._index] = self._solve_v(self._T , P)
-        self._liquid_mol[self._index] = self._mol_vle - self._v
+        set_flows(self._vapor_mol, self._liquid_mol, self._index,
+                  self._solve_v(self._T, P), self._mol_vle)
         self._H_hat = self.mixture.xH(self._phase_data, self._T, P)/self._F_mass
         return self._H_hat - H_hat
     
     def _S_hat_err_at_T(self, T, S_hat):
-        self._vapor_mol[self._index] = self._solve_v(T, self._P)
-        self._liquid_mol[self._index] = self._mol_vle - self._v
+        set_flows(self._vapor_mol, self._liquid_mol, self._index,
+                  self._solve_v(T, self._P), self._mol_vle)
         self._S_hat = self.mixture.xS(self._phase_data, T, self._P)/self._F_mass
         return self._S_hat - S_hat
     
     def _S_hat_err_at_P(self, P, S_hat):
-        self._vapor_mol[self._index] = self._solve_v(self._T , P)
-        self._liquid_mol[self._index] = self._mol_vle - self._v
+        set_flows(self._vapor_mol, self._liquid_mol, self._index,
+                  self._solve_v(self._T, P), self._mol_vle)
         self._S_hat = self.mixture.xS(self._phase_data, self._T, P)/self._F_mass
         return self._S_hat - S_hat
     
@@ -1174,24 +1195,23 @@ class VLE(Equilibrium, phases='lg'):
     def _V_err_at_T(self, T, V):
         return self._solve_v(T, self._P).sum()/self._F_mol_vle  - V
     
-    def _y_iter(self, y, Psats_over_P, T, P):
+    def _y_iter(self, y, Psats, Psats_over_P, T, P):
         phi = self._phi(y, T, P)
         gamma = self._gamma
-        pcf = self._pcf
         x = self._x
-        Psat_over_P_phi = Psats_over_P / phi
+        pcf_Psat_over_P_phi = self._pcf(T, P, Psats) * Psats_over_P / phi
         N = self._N
         z = self._z
         if N > 3 or self._z_light or self._z_heavy:
             f = xV_iter
-            args = (Psat_over_P_phi, T, z, self._z_light, 
-                    self._z_heavy, gamma.f, gamma.args, pcf.f, pcf.args)
+            args = (pcf_Psat_over_P_phi, T, P, z, self._z_light, 
+                    self._z_heavy, gamma.f, gamma.args)
         elif N == 2:
             f = xV_iter_2n
-            args = (Psat_over_P_phi, T, z, gamma.f, gamma.args, pcf.f, pcf.args)
+            args = (pcf_Psat_over_P_phi, T, P, z, gamma.f, gamma.args)
         elif N == 3:
             f = xV_iter_3n
-            args = (Psat_over_P_phi, T, z, gamma.f, gamma.args, pcf.f, pcf.args)
+            args = (pcf_Psat_over_P_phi, T, P, z, gamma.f, gamma.args)
         xV = np.zeros(x.size + 1)
         xV[:-1] = x
         xV[-1] = self._V
@@ -1207,7 +1227,7 @@ class VLE(Equilibrium, phases='lg'):
         else:
             Ks = (z / x - 1) / V + 1.
         self._z_last = z
-        v = self._F_mol * V * x * Ks     
+        v = self._F_mol * V * x * Ks    
         return fn.normalize(v, v.sum() + self._F_mol_light)
     
     def _solve_v(self, T, P):
@@ -1215,26 +1235,27 @@ class VLE(Equilibrium, phases='lg'):
         method = self.method
         if method == 'differential-evolution':
             gamma = self._gamma
-            pcf = self._pcf
             phi = self._phi
             Psats = np.array([i(T) for i in self._bubble_point.Psats]) 
+            pcf = self._pcf(T, P, Psats)
             F_mol_vle = self._F_mol_vle
             mol_vle = self._mol_vle
             z = mol_vle / F_mol_vle
             v = F_mol_vle * solve_vle_vapor_mol_differential_evolution(
-                z, T, gamma.f, gamma.args, pcf.f, pcf.args, P, Psats, phi.f, phi.args, 
+                z, T, gamma.f, gamma.args, pcf, P, Psats, phi.f, phi.args, 
                 **self.differential_evolution_options,
             )
             self._z_last = z
         elif method == 'fixed-point':
-            Psats_over_P = np.array([i(T) for i in
-                                     self._bubble_point.Psats]) / P
+            Psats = np.array([i(T) for i in
+                              self._bubble_point.Psats])
+            Psats_over_P = Psats / P
             self._T = T
             if isinstance(self._phi, IdealFugacityCoefficients):
-                y = self._y_iter(self._y, Psats_over_P, T, P)
+                y = self._y_iter(self._y, Psats, Psats_over_P, T, P)
             else:
                 y = flx.aitken(self._y_iter, self._y, self.y_tol,
-                               args=(Psats_over_P, T, P),
+                               args=(Psats, Psats_over_P, T, P),
                                checkiter=False, 
                                checkconvergence=False, 
                                convergenceiter=5,
@@ -1251,11 +1272,11 @@ class VLECache(Cache): load = VLE
 del Cache, Equilibrium
 
 @njit(cache=True)
-def liquid_fugacity(mol_L, T, Psats, f_gamma, gamma_args, f_pcf, pcf_args):
+def liquid_fugacity(mol_L, T, pcf_Psats, f_gamma, gamma_args):
     total_mol_L = mol_L.sum()
     if total_mol_L:
         x = mol_L / total_mol_L
-        fugacity = x * f_gamma(x, T, *gamma_args) * Psats * f_pcf(x, T, *pcf_args)
+        fugacity = x * f_gamma(x, T, *gamma_args) * pcf_Psats
     else:
         fugacity = np.ones_like(mol_L)
     return fugacity 
@@ -1277,20 +1298,20 @@ def gibbs_free_energy(mol, fugacity):
     return g_mix
 
 @njit(cache=True)
-def vle_objective_function(mol_v, mol, T, f_gamma, gamma_args, f_pcf, pcf_args, P, Psats, f_phi, phi_args):
+def vle_objective_function(mol_v, mol, T, f_gamma, gamma_args, P, pcf_Psats, f_phi, phi_args):
     mol_l = mol - mol_v
-    g_mix_l = gibbs_free_energy(mol_l, liquid_fugacity(mol_l, T, Psats, f_gamma, gamma_args, f_pcf, pcf_args))
+    g_mix_l = gibbs_free_energy(mol_l, liquid_fugacity(mol_l, T, pcf_Psats, f_gamma, gamma_args))
     g_mix_g = gibbs_free_energy(mol_v, vapor_fugacity(mol_v, T, P, f_phi, phi_args))
     g_mix = g_mix_l + g_mix_g
     return g_mix
 
 def solve_vle_vapor_mol_differential_evolution(
-        mol, T, f_gamma, gamma_args, f_pcf, pcf_args, P, Psats, f_phi, phi_args, 
+        mol, T, f_gamma, gamma_args, P, pcf_Psats, f_phi, phi_args, 
         **differential_evolution_options
     ):
-    args = (mol, T, f_gamma, gamma_args, f_pcf, pcf_args, P, Psats, f_phi, phi_args)
+    args = (mol, T, f_gamma, gamma_args, P, pcf_Psats, f_phi, phi_args)
     bounds = np.zeros([mol.size, 2])
     bounds[:, 1] = mol
     result = differential_evolution(vle_objective_function, bounds, args,
                                     **differential_evolution_options)
-    return result.x
\ No newline at end of file
+    return result.x
