@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # BioSTEAM: The Biorefinery Simulation and Techno-Economic Analysis Modules
-# Copyright (C) 2020-2021, Yoel Cortes-Pena <yoelcortes@gmail.com>
+# Copyright (C) 2020-2023, Yoel Cortes-Pena <yoelcortes@gmail.com>
 # 
 # This module is under the UIUC open-source license. See 
 # github.com/BioSTEAMDevelopmentGroup/biosteam/blob/master/LICENSE.txt
@@ -8,6 +8,7 @@
 """
 """
 import thermosteam as tmo
+from ._thermal_condition import mock_thermal_condition
 from .units_of_measure import AbsoluteUnitsOfMeasure
 from . import utils
 from .exceptions import UndefinedChemicalAlias
@@ -16,6 +17,7 @@ from ._phase import Phase, LockedPhase, NoPhase, PhaseIndexer, phase_tuple
 from free_properties import PropertyFactory, property_array
 from collections.abc import Iterable
 import numpy as np
+from numba import njit
 
 __all__ = (
     'SplitIndexer',
@@ -39,6 +41,24 @@ phase_names = {
     'S': 'SOLID',
 }
 
+# %% Numba acceleration
+
+@njit(cache=True)
+def sum_chemical_index(data, other, left_index, right_index):
+    data[left_index] += other[right_index]
+
+@njit(cache=True)
+def sum_material_index(data, other, left_index, right_index):
+    data[:, left_index] += other[:, right_index]
+    
+@njit(cache=True)
+def copy_chemical_index(data, other, left_index, right_index):
+    data[left_index] = other[right_index]
+
+@njit(cache=True)
+def copy_material_index(data, other, left_index, right_index):
+    data[:, left_index] = other[:, right_index]
+
 # %% Utilities
 
 _new = object.__new__
@@ -62,6 +82,35 @@ def nonzeros(IDs, data):
     index, = np.where(data != 0)
     return [IDs[i] for i in index], data[index]
 
+def index_overlap(left_chemicals, right_chemicals, right_data):
+    right_index, = np.where(right_data)
+    CASs_all = right_chemicals.CASs
+    CASs = tuple([CASs_all[i] for i in right_index])
+    cache = left_chemicals._index_cache
+    if CASs in cache:
+        left_index, kind = cache[CASs]
+        if kind:
+            raise RuntimeError('conflict in chemical groups and aliases between property packages')
+        else:
+            return left_index, right_index
+    else:
+        dct = left_chemicals._index
+        N = len(CASs)
+        left_index = np.zeros(N, dtype=int)
+        isa = isinstance
+        ndarray = np.ndarray
+        for i in range(N):
+            CAS = CASs[i]
+            if CAS in dct:
+                index = dct[CAS]
+                if isa(index, ndarray): raise RuntimeError('conflict in chemical groups and aliases between property packages')
+                left_index[i] = index
+            else:
+                raise UndefinedChemicalAlias(CAS)
+        cache[CASs] = (left_index, 0)
+        if len(cache) > 1000: cache.pop(cache.__iter__().__next__())
+        return left_index, right_index
+
 # %% Abstract indexer
     
 class Indexer:
@@ -73,7 +122,7 @@ class Indexer:
         self._data[:] = 0
     
     def isempty(self):
-        return (self._data == 0.).all()
+        return (self._data == 0.).all().__bool__()
     
     def copy(self):
         new = self._copy_without_data()
@@ -316,8 +365,9 @@ class ChemicalIndexer(Indexer):
             arr = np.zeros(len(index))
             data = self._data
             isa = isinstance
+            ndarray = np.ndarray
             for d, s in enumerate(index):
-                if isa(s, list): 
+                if isa(s, ndarray): 
                     arr[d] = data[s].sum()
                 else:
                     arr[d] = data[s]
@@ -336,9 +386,10 @@ class ChemicalIndexer(Indexer):
             local_data = self._data
             isa = isinstance
             group_compositions = self.group_compositions
+            ndarray = np.ndarray
             for n in range(len(index)):
                 i = index[n]
-                local_data[i] = data[n] * group_compositions[key[n]] if isa(i, list) else data[n]
+                local_data[i] = data[n] * group_compositions[key[n]] if isa(i, ndarray) else data[n]
         else:
             raise IndexError('unknown error')
     
@@ -353,27 +404,37 @@ class ChemicalIndexer(Indexer):
         self.phase = find_main_phase(others, self.phase)
         chemicals = self._chemicals
         data = self._data
-        chemicals_data = [(i._chemicals, i._data.copy() if i is self else i.sum_across_phases())
-                          for i in others]
-        data[:] = 0.
-        for ichemicals, idata in chemicals_data:
-            if chemicals is ichemicals:
-                data[:] += idata
+        sc_data = [] # Same chemicals
+        other_data = [] # Different chemicals
+        repeated_data = 0
+        base = data.base
+        for i in others:
+            if i is self:
+                repeated_data += 1
             else:
-                other_index, = np.where(idata)
-                CASs = ichemicals.CASs
-                self_index = chemicals.indices([CASs[i] for i in other_index])
-                data[self_index] += idata[other_index]
+                idata = i.sum_across_phases()
+                if idata.base is base or idata is base: idata = idata.copy()
+                ichemicals = i._chemicals
+                if ichemicals is chemicals:
+                    sc_data.append(idata)
+                else:
+                    other_data.append(
+                        (idata, *index_overlap(chemicals, ichemicals, idata))
+                    )
+        if repeated_data == 0:
+            data[:] = 0.
+        elif repeated_data > 1:
+            data[:] *= repeated_data
+        for i in sc_data: data[:] += i
+        for args in other_data: sum_chemical_index(data, *args)
     
     def separate_out(self, other):
         if self._chemicals is other._chemicals:
             self._data[:] -= other.sum_across_phases()
         else:
-            idata = other._data
-            other_index, = np.where(idata)
-            IDs = other._chemicals.IDs
-            self_index = self._chemicals.indices([IDs[i] for i in other_index])
-            self._data[self_index] -= idata[other_index]
+            other_data = other._data
+            left_index, right_index = index_overlap(self._chemicals, other._chemicals, other_data)
+            self._data[left_index] -= other_data[right_index]
     
     def to_material_indexer(self, phases):
         material_array = self._MaterialIndexer.blank(phases, self._chemicals)
@@ -386,10 +447,8 @@ class ChemicalIndexer(Indexer):
             self._data[:] = other._data
         else:
             self.empty()
-            other_index, = np.where(other._data)
-            CASs = other.chemicals.CASs
-            self_index = self.chemicals.indices([CASs[i] for i in other_index])
-            self._data[self_index] = other._data[other_index]
+            other_data = other._data
+            copy_chemical_index(self._data, other_data, *index_overlap(self._chemicals, other._chemicals, other_data))
         self.phase = other.phase
     
     def _copy_without_data(self):
@@ -564,20 +623,15 @@ class MaterialIndexer(Indexer):
             if self.chemicals is other.chemicals:
                 self._data[phase_index, :] = other_data
             else:
-                other_index, = np.where(other_data)
-                IDs = other.chemicals.IDs
-                self_index = self.chemicals.indices([IDs[i] for i in other_index])
-                self._data[phase_index, self_index] += other._data[other_index]
+                other_data = other._data
+                copy_chemical_index(self._data[phase_index], other_data, *index_overlap(self._chemicals, other._chemicals, other_data))
         else:
             if self.chemicals is other.chemicals:
                 self._data[:] = other._data
             else:
                 self.empty()
                 other_data = other._data
-                other_index, = np.where(other_data.any(0))
-                IDs = other.chemicals.IDs
-                self_index = self.chemicals.indices([IDs[i] for i in other_index])
-                self._data[:, self_index] = other_data[:, other_index]
+                copy_material_index(self._data, other_data, *index_overlap(self._chemicals, other._chemicals, other_data.any(0)))
     
     def mix_from(self, others):
         isa = isinstance
@@ -585,41 +639,59 @@ class MaterialIndexer(Indexer):
         get_phase_index = self.get_phase_index
         chemicals = self._chemicals
         phases = self._phases
-        indexer_data = [(i, i._data.copy() if i is self else i._data) for i in others]
-        data[:] = 0.
-        for i, idata in indexer_data:
-            if isa(i, MaterialIndexer):
-                if phases == i.phases:
-                    if chemicals is i.chemicals:
-                        data[:] += idata
-                    else:
-                        idata = i._data
-                        other_index, = np.where(idata.any(0))
-                        IDs = i.chemicals.IDs
-                        self_index = chemicals.indices([IDs[i] for i in other_index])
-                        data[:, self_index] += idata[:, other_index]
-                else:
-                    if chemicals is i.chemicals:
-                        for phase, idata in zip(i.phases, idata):
-                            if not idata.any(): continue
-                            data[get_phase_index(phase), :] += idata
-                    else:
-                        for phase, idata in zip(i.phases, idata):
-                            if not idata.any(): continue
-                            other_index, = np.where(idata)
-                            IDs = i.chemicals.IDs
-                            self_index = chemicals.indices([IDs[i] for i in other_index])
-                            data[get_phase_index(phase), self_index] += idata[other_index]
-            elif isa(i, ChemicalIndexer):
-                if chemicals is i.chemicals:
-                    data[get_phase_index(i.phase), :] += idata
-                else:
-                    other_index, = np.where(idata != 0.)
-                    IDs = i.chemicals.IDs
-                    self_index = chemicals.indices([IDs[i] for i in other_index])
-                    data[get_phase_index(i.phase), self_index] += idata[other_index]
+        repeated_data = 0
+        spsc_data = [] # Same phases, same chemicals
+        sp_data = [] # Same phases, different chemicals
+        opsc_data = [] # One phase, same chemicals
+        op_data = [] # One phase, different chemicals
+        for i in others:
+            if i is self:
+                repeated_data += 1
             else:
-                raise ValueError("can only mix from chemical or material indexers")
+                idata = i._data
+                if idata.base is data: idata = idata.copy()
+                ichemicals = i._chemicals
+                if isa(i, MaterialIndexer):
+                    if phases == i.phases:
+                        if chemicals is ichemicals:
+                            spsc_data.append(idata)
+                        else:
+                            sp_data.append(
+                                (idata, *index_overlap(chemicals, ichemicals, idata.any(0)))
+                            )
+                    else:
+                        if chemicals is ichemicals:
+                            for phase, idata in zip(i.phases, idata):
+                                if not idata.any(): continue
+                                opsc_data.append(
+                                    (idata, get_phase_index(i.phase))
+                                )
+                        else:
+                            for phase, idata in zip(i.phases, idata):
+                                if not idata.any(): continue
+                                op_data.append(
+                                    (get_phase_index(phase), (idata, *index_overlap(chemicals, ichemicals, idata)))
+                                )
+                elif isa(i, ChemicalIndexer):
+                    if idata.base is data: idata = idata.copy()
+                    if chemicals is ichemicals:
+                        opsc_data.append(
+                            (idata, get_phase_index(i.phase))
+                        )
+                    else:
+                        op_data.append(
+                            (get_phase_index(phase), (idata, *index_overlap(chemicals, ichemicals, idata)))
+                        )
+                else:
+                    raise ValueError("can only mix from chemical or material indexers")
+        if repeated_data == 0:
+            data[:] = 0.
+        elif repeated_data > 1:
+            data[:] *= repeated_data
+        for i in spsc_data: data[:] += i
+        for args in sp_data: sum_material_index(data, *args)
+        for idata, left_index in opsc_data: data[left_index, :] += idata
+        for phase_index, args in op_data: sum_chemical_index(data[phase_index], *args)
     
     def separate_out(self, other):
         isa = isinstance
@@ -635,8 +707,8 @@ class MaterialIndexer(Indexer):
                 else:
                     idata = other._data
                     other_index, = np.where(idata.any(0))
-                    IDs = other.chemicals.IDs
-                    self_index = chemicals.indices([IDs[i] for i in other_index])
+                    CASs = other.chemicals.CASs
+                    self_index = chemicals.indices([CASs[i] for i in other_index])
                     data[:, self_index] -= idata[:, other_index]
             else:
                 if chemicals is other.chemicals:
@@ -647,16 +719,16 @@ class MaterialIndexer(Indexer):
                     for phase, idata in zip(other.phases, idata):
                         if not idata.any(): continue
                         other_index, = np.where(idata)
-                        IDs = other.chemicals.IDs
-                        self_index = chemicals.indices([IDs[i] for i in other_index])
+                        CASs = other.chemicals.CASs
+                        self_index = chemicals.indices([CASs[i] for i in other_index])
                         data[get_phase_index(phase), self_index] -= idata[other_index]
         elif isa(other, ChemicalIndexer):
             if chemicals is other.chemicals:
                 data[get_phase_index(other.phase), :] -= idata
             else:
                 other_index, = np.where(idata != 0.)
-                IDs = other.chemicals.IDs
-                self_index = chemicals.indices([IDs[i] for i in other_index])
+                CASs = other.chemicals.CASs
+                self_index = chemicals.indices([CASs[i] for i in other_index])
                 data[get_phase_index(other.phase), self_index] -= idata[other_index]
         else:
             raise ValueError("can only separate out from chemical or material indexers")
@@ -757,17 +829,18 @@ class MaterialIndexer(Indexer):
                 data = self._data
                 isa = isinstance
                 phase, index = index
+                ndarray = np.ndarray
                 if phase == slice(None):
                     values = np.zeros([len(self.phases), len(index)])
                     for d, s in enumerate(index):
-                        if isa(s, list): 
+                        if isa(s, ndarray): 
                             values[:, d] = data[phase, s].sum(1)
                         else:
                             values[:, d] = data[phase, s]
                 else:
                     values = np.zeros(len(index))
                     for d, s in enumerate(index):
-                        if isa(s, list): 
+                        if isa(s, ndarray): 
                             values[d] = data[phase, s].sum()
                         else:
                             values[d] = data[phase, s]
@@ -790,9 +863,10 @@ class MaterialIndexer(Indexer):
             local_data = self._data
             group_compositions = self.group_compositions
             isa = isinstance
+            ndarray = np.ndarray
             for n in range(len(index)):
                 i = index[n]
-                local_data[phase, i] = data[n] * group_compositions[key[n]] if isa(i, list) else data[n]
+                local_data[phase, i] = data[n] * group_compositions[key[n]] if isa(i, ndarray) else data[n]
         else:
             raise IndexError('unknown error')
     
@@ -1038,22 +1112,33 @@ MolarFlowIndexer.by_mass = by_mass; del by_mass
 
 # %% Volumetric flow properties
 
+TP_V = (mock_thermal_condition, None) # Initial cache for molar volume
+
 @PropertyFactory(slots=('name', 'mol', 'index', 'V',
-                        'TP', 'phase', 'phase_container'),
+                        'TP', 'phase', 'phase_container', 'cache'),
                  units='m^3/hr')
 def VolumetricFlowProperty(self):
     """Volumetric flow (m^3/hr)."""
     f_mol = self.mol[self.index] 
-    phase = self.phase or self.phase_container.phase
-    V = getattr(self.V, phase) if isinstance(self.V, PhaseHandle) else self.V
-    return 1000. * f_mol * V(*self.TP) if f_mol else 0.
+    if f_mol:
+        TP, V = self.cache
+        if not TP.in_equilibrium(self.TP):
+            phase = self.phase or self.phase_container.phase
+            V = 1000. * (getattr(self.V, phase) if isinstance(self.V, PhaseHandle) else self.V)(*self.TP)
+            self.cache = (self.TP.copy(), V)
+        return f_mol * V
+    else:
+        return 0.
     
 @VolumetricFlowProperty.setter
 def VolumetricFlowProperty(self, value):
     if value:
-        phase = self.phase or self.phase_container.phase
-        V = getattr(self.V, phase) if isinstance(self.V, PhaseHandle) else self.V
-        self.mol[self.index] = value / V(*self.TP) / 1000.
+        TP, V = self.cache
+        if not TP.in_equilibrium(self.TP):
+            phase = self.phase or self.phase_container.phase
+            V = 1000. * (getattr(self.V, phase) if isinstance(self.V, PhaseHandle) else self.V)(*self.TP)
+            self.cache = (self.TP.copy(), V)
+        self.mol[self.index] = value / V
     else:
         self.mol[self.index] = 0.
 
@@ -1073,7 +1158,7 @@ def by_volume(self, TP):
         vol = np.zeros_like(mol, dtype=object)
         for i, chem in enumerate(chemicals):
             vol[i] = VolumetricFlowProperty(chem.ID, mol, i, chem.V,
-                                            TP, None, self._phase)
+                                            TP, None, self._phase, TP_V)
         self._data_cache['vol', TP] = \
         vol = ChemicalVolumetricFlowIndexer.from_data(property_array(vol),
                                                       self._phase, chemicals,
@@ -1101,7 +1186,8 @@ def by_volume(self, TP):
                 index = i, j
                 phase_name = phase_names[phase]
                 vol[index] = VolumetricFlowProperty(f"{phase_name}{chem.ID}", 
-                                                    mol, index, chem.V, TP, phase)
+                                                    mol, index, chem.V, TP, phase,
+                                                    None, TP_V)
         self._data_cache[TP] = \
         vol = VolumetricFlowIndexer.from_data(property_array(vol),
                                               phases, chemicals,
