@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # BioSTEAM: The Biorefinery Simulation and Techno-Economic Analysis Modules
-# Copyright (C) 2020-2021, Yoel Cortes-Pena <yoelcortes@gmail.com>
+# Copyright (C) 2020-2023, Yoel Cortes-Pena <yoelcortes@gmail.com>
 # 
 # This module is under the UIUC open-source license. See 
 # github.com/BioSTEAMDevelopmentGroup/biosteam/blob/master/LICENSE.txt
@@ -12,8 +12,9 @@ import numpy as np
 from math import exp
 from thermosteam import functional as fn
 from .. import units_of_measure as thermo_units
-from ..base import PhaseMixtureHandle
+from ..base import PhaseHandle, MockPhaseTHandle, MockPhaseTPHandle
 from .ideal_mixture_model import (
+    SinglePhaseIdealTMixtureModel,
     IdealTMixtureModel, 
     IdealTPMixtureModel, 
     IdealEntropyModel, 
@@ -25,32 +26,21 @@ __all__ = ('Mixture',)
 
 # %% Functions for building mixture models
 
-def group_handles_by_phase(phase_handles):
-    hasfield = hasattr
+def create_mixture_model(chemicals, var, Model):
     getfield = getattr
-    iscallable = callable
-    handles_by_phase = {'s': [],
-                        'l': [],
-                        'g': []}
-    for phase, handles in handles_by_phase.items():
-        for phase_handle in phase_handles:
-            if iscallable(phase_handle) and hasfield(phase_handle, phase):
-                prop = getfield(phase_handle, phase)
-                if not iscallable(prop): prop = phase_handle
-            else:
-                prop = phase_handle
-            handles.append(prop)
-    return handles_by_phase
+    isa = isinstance
+    handles = []
+    for chemical in chemicals:
+        obj = getfield(chemical, var)
+        if isa(obj, PhaseHandle):
+            phase_handle = obj
+        elif var == 'Cn':
+            phase_handle = MockPhaseTHandle(var, obj)
+        else:
+            phase_handle = MockPhaseTPHandle(var, obj)
+        handles.append(phase_handle)
+    return Model(handles, var)
     
-def build_ideal_PhaseMixtureHandle(chemicals, var, Model):
-    setfield = object.__setattr__
-    getfield = getattr
-    phase_handles = [getfield(i, var) for i in chemicals]
-    new = PhaseMixtureHandle.__new__(PhaseMixtureHandle)
-    for phase, handles in group_handles_by_phase(phase_handles).items():
-        setfield(new, phase, Model(handles, var))
-    setfield(new, 'var', var)
-    return new
 
 # %% Energy balance
 
@@ -157,6 +147,8 @@ class Mixture:
         Component molecular weights [g/mol].
     
     """
+    maxiter = 20
+    T_tol = 1e-6
     __slots__ = ('rule',
                  'rigorous_energy_balance',
                  'include_excess_energies',
@@ -233,17 +225,17 @@ class Mixture:
                 chemicals = [(i if isa(i, Chemical) else Chemical(i)) for i in chemicals]
                 MWs = chemical_data_array(chemicals, 'MW')
             getfield = getattr
-            Cn =  build_ideal_PhaseMixtureHandle(chemicals, 'Cn', IdealTMixtureModel)
-            H =  build_ideal_PhaseMixtureHandle(chemicals, 'H', IdealTMixtureModel)
-            S = build_ideal_PhaseMixtureHandle(chemicals, 'S', IdealEntropyModel)
-            H_excess = build_ideal_PhaseMixtureHandle(chemicals, 'H_excess', IdealTPMixtureModel)
-            S_excess = build_ideal_PhaseMixtureHandle(chemicals, 'S_excess', IdealTPMixtureModel)
-            mu = build_ideal_PhaseMixtureHandle(chemicals, 'mu', IdealTPMixtureModel)
-            V = build_ideal_PhaseMixtureHandle(chemicals, 'V', IdealTPMixtureModel)
-            kappa = build_ideal_PhaseMixtureHandle(chemicals, 'kappa', IdealTPMixtureModel)
+            Cn =  create_mixture_model(chemicals, 'Cn', IdealTMixtureModel)
+            H =  create_mixture_model(chemicals, 'H', IdealTPMixtureModel)
+            S = create_mixture_model(chemicals, 'S', IdealEntropyModel)
+            H_excess = create_mixture_model(chemicals, 'H_excess', IdealTPMixtureModel)
+            S_excess = create_mixture_model(chemicals, 'S_excess', IdealTPMixtureModel)
+            mu = create_mixture_model(chemicals, 'mu', IdealTPMixtureModel)
+            V = create_mixture_model(chemicals, 'V', IdealTPMixtureModel)
+            kappa = create_mixture_model(chemicals, 'kappa', IdealTPMixtureModel)
             Hvap = IdealHvapModel(chemicals)
-            sigma = IdealTMixtureModel([getfield(i, 'sigma') for i in chemicals], 'sigma')
-            epsilon = IdealTMixtureModel([getfield(i, 'epsilon') for i in chemicals], 'epsilon')
+            sigma = SinglePhaseIdealTMixtureModel([getfield(i, 'sigma') for i in chemicals], 'sigma')
+            epsilon = SinglePhaseIdealTMixtureModel([getfield(i, 'epsilon') for i in chemicals], 'epsilon')
             return cls(rule, Cn, H, S, H_excess, S_excess,
                        mu, V, kappa, Hvap, sigma, epsilon, MWs, include_excess_energies)
         else:
@@ -329,14 +321,14 @@ class Mixture:
     
     def H(self, phase, mol, T, P):
         """Return enthalpy [J/mol]."""
-        H = self._H(phase, mol, T)
+        H = self._H(phase, mol, T, P)
         if self.include_excess_energies: H += self._H_excess(phase, mol, T, P)
         return H
     
     def S(self, phase, mol, T, P):
         """Return entropy in [J/mol/K]."""
         total_mol = mol.sum()
-        if total_mol == 0. : return 0.
+        if total_mol == 0.: return 0.
         S = self._S(phase, mol, T, P)
         if self.include_excess_energies:
             S += self._S_excess(phase, mol, T, P)
@@ -345,24 +337,56 @@ class Mixture:
     def solve_T_at_HP(self, phase, mol, H, T_guess, P):
         """Solve for temperature in Kelvin."""
         args = (H, self.H, phase, mol, P, self.Cn, [0, None])
-        return flx.aitken(iter_T_at_HP, T_guess, 1e-6, args, 50, checkiter=True)
+        T_guess = flx.aitken(iter_T_at_HP, T_guess, self.T_tol, args, self.maxiter, checkiter=False)
+        T = iter_T_at_HP(T_guess, *args)
+        return (
+            flx.aitken_secant(
+                lambda T: self.H(phase, mol, T, P) - H,
+                x0=T_guess, x1=T, xtol=self.T_tol, ytol=0.
+            )
+            if abs(T - T_guess) < self.T_tol else T
+        )
         
     def xsolve_T_at_HP(self, phase_mol, H, T_guess, P):
         """Solve for temperature in Kelvin."""
         phase_mol = tuple(phase_mol)
         args = (H, self.xH, phase_mol, P, self.xCn, [0, None])
-        return flx.aitken(xiter_T_at_HP, T_guess, 1e-6, args, 50, checkiter=True)
+        T_guess = flx.aitken(xiter_T_at_HP, T_guess, self.T_tol, args, self.maxiter, checkiter=False)
+        T = xiter_T_at_HP(T_guess, *args)
+        return (
+            flx.aitken_secant(
+                lambda T: self.xH(phase_mol, T, P) - H,
+                x0=T_guess, x1=T, xtol=self.T_tol, ytol=0.
+            )
+            if abs(T - T_guess) < self.T_tol else T
+        )
     
     def solve_T_at_SP(self, phase, mol, S, T_guess, P):
         """Solve for temperature in Kelvin."""
         args = (S, self.S, phase, mol, P, self.Cn, [0, None])
-        return flx.aitken(iter_T_at_SP, T_guess, 1e-6, args, 50, checkiter=True)
+        T_guess = flx.aitken(iter_T_at_SP, T_guess, self.T_tol, args, self.maxiter, checkiter=False)
+        T = iter_T_at_SP(T_guess, *args)
+        return (
+            flx.aitken_secant(
+                lambda T: self.S(phase, mol, T, P) - S,
+                x0=T_guess, x1=T, xtol=self.T_tol, ytol=0.
+            )
+            if abs(T - T_guess) < self.T_tol else T
+        )
         
     def xsolve_T_at_SP(self, phase_mol, S, T_guess, P):
         """Solve for temperature in Kelvin."""
         phase_mol = tuple(phase_mol)
         args = (S, self.xS, phase_mol, P, self.xCn, [0, None])
-        return flx.aitken(xiter_T_at_SP, T_guess, 1e-6, args, 50, checkiter=True)
+        T_guess = flx.aitken(xiter_T_at_SP, T_guess, self.T_tol, args, self.maxiter, checkiter=False)
+        T = xiter_T_at_SP(T_guess, *args)
+        return (
+            flx.aitken_secant(
+                lambda T: self.xS(phase_mol, T, P) - S,
+                x0=T_guess, x1=T, xtol=self.T_tol, ytol=0.
+            )
+            if abs(T - T_guess) < self.T_tol else T
+        )
     
     def xCn(self, phase_mol, T, P=None):
         """Multi-phase mixture molar isobaric heat capacity [J/mol/K]."""
@@ -371,7 +395,8 @@ class Mixture:
     def xH(self, phase_mol, T, P):
         """Multi-phase mixture enthalpy [J/mol]."""
         H = self._H
-        H_total = sum([H(phase, mol, T) for phase, mol in phase_mol])
+        phase_mol = tuple(phase_mol)
+        H_total = sum([H(phase, mol, T, P) for phase, mol in phase_mol])
         if self.include_excess_energies:
             H_excess = self._H_excess
             H_total += sum([H_excess(phase, mol, T, P) for phase, mol in phase_mol])
@@ -380,6 +405,7 @@ class Mixture:
     def xS(self, phase_mol, T, P):
         """Multi-phase mixture entropy [J/mol/K]."""
         S = self._S
+        phase_mol = tuple(phase_mol)
         S_total = sum([S(phase, mol, T, P) for phase, mol in phase_mol])
         if self.include_excess_energies:
             S_excess = self._S_excess
