@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # BioSTEAM: The Biorefinery Simulation and Techno-Economic Analysis Modules
-# Copyright (C) 2020-2021, Yoel Cortes-Pena <yoelcortes@gmail.com>
+# Copyright (C) 2020-2023, Yoel Cortes-Pena <yoelcortes@gmail.com>
 # 
 # This module is under the UIUC open-source license. See 
 # github.com/BioSTEAMDevelopmentGroup/biosteam/blob/master/LICENSE.txt
@@ -13,17 +13,20 @@ from . import read_only
 from . import registered
 from . import forward
 from . import units_of_measure
+from . import chemical_cache
 
 __all__ = (*chemicals_user.__all__,
            *thermo_user.__all__,
            *read_only.__all__,
            *registered.__all__,
            *forward.__all__,
-           *units_of_measure.__all__)
+           *units_of_measure.__all__,
+           *chemical_cache.__all__)
 
 from .registered import *
 from .chemicals_user import *
 from .thermo_user import *
 from .read_only import *
 from .forward import *
-from .units_of_measure import *
\ No newline at end of file
+from .units_of_measure import *
+from .chemical_cache import *
